@@ -1,5 +1,5 @@
 /******************************************************************************
- * $Id: proxy.c 3401 2007-10-13 17:33:53Z charles $
+ * $Id: proxy.c 4390 2007-12-31 01:19:35Z livings124 $
  *
  * Copyright (c) 2007 Joshua Elsasser
  *
@@ -128,7 +128,7 @@ usage( const char * msg, ... )
     printf(
   "usage: %s [options] [files]...\n"
   "\n"
-  "Transmission %s http://transmission.m0k.org/\n"
+  "Transmission %s http://www.transmissionbt.com/\n"
   "A fast and easy BitTorrent client\n"
   "\n"
   "  -h --help                 Display this message and exit\n"
