@@ -1,5 +1,5 @@
 /******************************************************************************
- * $Id: remote.c 3401 2007-10-13 17:33:53Z charles $
+ * $Id: remote.c 4908 2008-02-03 07:03:39Z charles $
  *
  * Copyright (c) 2007 Joshua Elsasser
  *
@@ -72,6 +72,7 @@ struct opts
     struct strlist    remove;
     char              dir[MAXPATHLEN];
     int               pex;
+    const char *      crypto;
 };
 
 struct torinfo
@@ -174,6 +175,7 @@ main( int argc, char ** argv )
     if( ( o.sendquit                &&   0 > client_quit     (           ) ) ||
         ( '\0' != o.dir[0]          &&   0 > client_dir      ( o.dir     ) ) ||
         ( !SLIST_EMPTY( &o.files )  &&   0 > client_addfiles ( &o.files  ) ) ||
+        ( o.crypto                  &&   0 > client_crypto   ( o.crypto  ) ) ||
         ( o.startall                &&   0 > client_start    ( 0, NULL   ) ) ||
         ( o.stopall                 &&   0 > client_stop     ( 0, NULL   ) ) ||
         ( o.removeall               &&   0 > client_remove   ( 0, NULL   ) ) ||
@@ -223,12 +225,15 @@ usage( const char * msg, ... )
     }
 
     printf(
-  "usage: %s [options]\n"
+  "usage: %s [options] [files...]\n"
+  "       %s -x [options] proxy-command [args...]\n"
   "\n"
-  "Transmission %s http://transmission.m0k.org/\n"
+  "Transmission %s http://www.transmissionbt.com/\n"
   "A fast and easy BitTorrent client\n"
   "\n"
   "  -a --add <torrent>        Add a torrent\n"
+  "  -c --encryption preferred Prefer peers to use encryption\n"
+  "  -c --encryption required  Require encryption for all peers\n"
   "  -d --download-limit <int> Max download rate in KiB/s\n"
   "  -D --download-unlimited   No download rate limit\n"
   "  -e --enable-pex           Enable peer exchange\n"
@@ -253,17 +258,18 @@ usage( const char * msg, ... )
   "  -u --upload-limit <int>   Max upload rate in KiB/s\n"
   "  -U --upload-unlimited     No upload rate limit\n"
   "  -x --proxy                Use proxy command to connect to frontend\n",
-            getmyname(), LONG_VERSION_STRING );
+            getmyname(), getmyname(), LONG_VERSION_STRING );
     exit( 0 );
 }
 
 int
 readargs( int argc, char ** argv, struct opts * opts )
 {
-    char optstr[] = "a:d:DeEf:hilmMp:qr:s:S:t:u:Ux";
+    char optstr[] = "a:c:d:DeEf:hilmMp:qr:s:S:t:u:Ux";
     struct option longopts[] =
     {
         { "add",                required_argument, NULL, 'a' },
+        { "encryption",         required_argument, NULL, 'c' },
         { "download-limit",     required_argument, NULL, 'd' },
         { "download-unlimited", no_argument,       NULL, 'D' },
         { "enable-pex",         no_argument,       NULL, 'e' },
@@ -282,7 +288,7 @@ readargs( int argc, char ** argv, struct opts * opts )
         { "type",               required_argument, NULL, 't' },
         { "upload-limit",       required_argument, NULL, 'u' },
         { "upload-unlimited",   no_argument,       NULL, 'U' },
-        { "proxy",              no_argument,       NULL, 'U' },
+        { "proxy",              no_argument,       NULL, 'x' },
         { NULL, 0, NULL, 0 }
     };
     int opt, gotmsg;
@@ -307,6 +313,14 @@ readargs( int argc, char ** argv, struct opts * opts )
                     return -1;
                 }
                 break;
+            case 'c':
+                if(!strcasecmp(optarg, "preferred"))
+                    opts->crypto = "preferred";
+                else if(!strcasecmp(optarg, "required"))
+                    opts->crypto = "required";
+                else
+                    usage("invalid encryption mode: %s", optarg);
+                break;
             case 'd':
                 opts->downlimit = 1;
                 opts->down      = numarg( optarg );
@@ -398,7 +412,7 @@ readargs( int argc, char ** argv, struct opts * opts )
                 break;
             case 'x':
                 opts->proxy     = 1;
-                break;
+                continue; /* don't set gotmsg, -x isn't a message */
             default:
                 usage( NULL );
                 break;
@@ -413,6 +427,8 @@ readargs( int argc, char ** argv, struct opts * opts )
 
     if( opts->proxy )
     {
+        if( argc == optind )
+            usage( "can't use -x without any arguments" );
         opts->proxycmd = argv + optind;
     }
     else if( 0 > fileargs( &opts->files, argc - optind, argv + optind ) )
