@@ -1,5 +1,5 @@
 /******************************************************************************
- * $Id: server.c 3579 2007-10-26 04:15:33Z joshe $
+ * $Id: server.c 4305 2007-12-24 03:48:42Z livings124 $
  *
  * Copyright (c) 2007 Joshua Elsasser
  *
@@ -104,10 +104,12 @@ server_init( struct event_base * base )
         0 > ipc_addmsg( gl_tree, IPC_MSG_ADDONEFILE,   addmsg2 ) ||
         0 > ipc_addmsg( gl_tree, IPC_MSG_AUTOMAP,      intmsg  ) ||
         0 > ipc_addmsg( gl_tree, IPC_MSG_AUTOSTART,    intmsg  ) ||
+        0 > ipc_addmsg( gl_tree, IPC_MSG_CRYPTO,       strmsg  ) ||
         0 > ipc_addmsg( gl_tree, IPC_MSG_DOWNLIMIT,    intmsg  ) ||
         0 > ipc_addmsg( gl_tree, IPC_MSG_DIR,          strmsg  ) ||
         0 > ipc_addmsg( gl_tree, IPC_MSG_GETAUTOMAP,   prefmsg ) ||
         0 > ipc_addmsg( gl_tree, IPC_MSG_GETAUTOSTART, prefmsg ) ||
+        0 > ipc_addmsg( gl_tree, IPC_MSG_GETCRYPTO,    prefmsg ) ||
         0 > ipc_addmsg( gl_tree, IPC_MSG_GETDOWNLIMIT, prefmsg ) ||
         0 > ipc_addmsg( gl_tree, IPC_MSG_GETDIR,       prefmsg ) ||
         0 > ipc_addmsg( gl_tree, IPC_MSG_GETINFO,      infomsg ) ||
@@ -635,6 +637,19 @@ strmsg( enum ipc_msg id, benc_val_t * val, int64_t tag, void * arg )
 
     switch( id )
     {
+        case IPC_MSG_CRYPTO:
+            if(!strcasecmp(val->val.s.s, "plaintext"))
+                torrent_set_encryption(TR_PLAINTEXT_PREFERRED);
+            else if(!strcasecmp(val->val.s.s, "preferred"))
+                torrent_set_encryption(TR_ENCRYPTION_PREFERRED);
+            else if(!strcasecmp(val->val.s.s, "required"))
+                torrent_set_encryption(TR_ENCRYPTION_REQUIRED);
+            else
+            {
+                msgresp(client, tag, IPC_MSG_BAD);
+                return;
+            }
+            break;
         case IPC_MSG_DIR:
             torrent_set_directory( val->val.s.s );
             break;
@@ -902,6 +917,7 @@ prefmsg( enum ipc_msg id, benc_val_t * val UNUSED, int64_t tag, void * arg )
     struct client * client = arg;
     uint8_t       * buf;
     size_t          buflen;
+    const char    * strval;
 
     switch( id )
     {
@@ -913,6 +929,24 @@ prefmsg( enum ipc_msg id, benc_val_t * val UNUSED, int64_t tag, void * arg )
             buf = ipc_mkint( client->ipc, &buflen, IPC_MSG_AUTOSTART, tag,
                              torrent_get_autostart() );
             break;
+        case IPC_MSG_GETCRYPTO:
+            switch(torrent_get_encryption())
+            {
+                case TR_PLAINTEXT_PREFERRED:
+                    strval = "plaintext";
+                    break;
+                case TR_ENCRYPTION_PREFERRED:
+                    strval = "preferred";
+                    break;
+                case TR_ENCRYPTION_REQUIRED:
+                    strval = "required";
+                    break;
+                default:
+                    assert(0);
+                    return;
+            }
+            buf = ipc_mkstr(client->ipc, &buflen, IPC_MSG_CRYPTO, tag, strval);
+            break;
         case IPC_MSG_GETDIR:
             buf = ipc_mkstr( client->ipc, &buflen, IPC_MSG_DIR, tag,
                              torrent_get_directory() );
