@@ -1,5 +1,5 @@
 /*
- * This file Copyright (C) 2007 Charles Kerr <charles@rebelbase.com>
+ * This file Copyright (C) 2007-2008 Charles Kerr <charles@rebelbase.com>
  *
  * This file is licensed by the GPL version 2.  Works owned by the
  * Transmission project are granted a special exemption to clause 2(b)
@@ -7,14 +7,17 @@
  * This exemption does not extend to derived works not owned by
  * the Transmission project.
  * 
- * $Id: actions.c 3392 2007-10-13 13:51:38Z charles $
+ * $Id: actions.c 4969 2008-02-07 20:27:15Z charles $
  */
 
 #include <string.h>
 #include <glib/gi18n.h>
 #include <gtk/gtk.h>
 #include <libtransmission/transmission.h>
+#include "conf.h"
 #include "torrent-inspector.h"
+#include "tr_core.h"
+#include "tr_prefs.h"
 #include "lock.h"
 #include "logo.h"
 
@@ -22,9 +25,12 @@
 
 extern void doAction (const char * action_name, gpointer user_data );
 
+static TrCore * myCore = 0;
+
 static GtkActionGroup * myGroup = 0;
 
-static void action_cb ( GtkAction * a, gpointer user_data )
+static void
+action_cb ( GtkAction * a, gpointer user_data )
 {
   doAction ( gtk_action_get_name(a), user_data );
 }
@@ -53,29 +59,76 @@ priority_changed_cb (GtkAction *action UNUSED, GtkRadioAction *current)
   set_selected_file_priority ( priority );
 }
 
+static GtkRadioActionEntry sort_radio_entries[] =
+{
+  { "sort-by-activity",   NULL, N_("Sort by _Activity"),   NULL, NULL, 0 },
+  { "sort-by-date-added", NULL, N_("Sort by _Date Added"), NULL, NULL, 1 },
+  { "sort-by-name",       NULL, N_("Sort by _Name"),       NULL, NULL, 2 },
+  { "sort-by-progress",   NULL, N_("Sort by _Progress"),   NULL, NULL, 3 },
+  { "sort-by-state",      NULL, N_("Sort by _State"),      NULL, NULL, 4 },
+  { "sort-by-tracker",    NULL, N_("Sort by _Tracker"),    NULL, NULL, 5 }
+};
+
+static void
+sort_changed_cb( GtkAction            * action UNUSED,
+                 GtkRadioAction       * current,
+                 gpointer user_data     UNUSED )
+{
+    const char * key = PREF_KEY_SORT_MODE;
+    const int i = gtk_radio_action_get_current_value( current );
+    const char * val = sort_radio_entries[i].name;
+    tr_core_set_pref( myCore, key, val );
+}
+
 static GtkToggleActionEntry show_toggle_entries[] = 
 {
   { "toggle-main-window", NULL, 
     N_("Show _Main Window"), NULL, NULL, G_CALLBACK(action_cb), TRUE }, 
-  { "toggle-debug-window", NULL,
-    N_("Show _Debug Window"), NULL, NULL, G_CALLBACK(action_cb), FALSE }
+  { "toggle-message-log", NULL,
+    N_("Show Message _Log"), NULL, NULL, G_CALLBACK(action_cb), FALSE }
+};
+
+static void
+toggle_pref_cb ( GtkToggleAction * action, gpointer user_data UNUSED )
+{
+    const char * key = gtk_action_get_name( GTK_ACTION( action ) );
+    const gboolean val = gtk_toggle_action_get_active( action );
+    tr_core_set_pref_bool( myCore, key, val );
+}
+
+static GtkToggleActionEntry pref_toggle_entries[] =
+{
+  { "minimal-view", NULL,
+    N_("_Minimal View"), "<alt>M", NULL, G_CALLBACK(toggle_pref_cb), FALSE },
+  { "sort-reversed", NULL,
+    N_("_Reverse Sort Order"), NULL, NULL, G_CALLBACK(toggle_pref_cb), FALSE },
+  { "show-filter-bar", NULL,
+    N_("Show _Filter Bar"), NULL, NULL, G_CALLBACK(toggle_pref_cb), FALSE },
+  { "show-status-bar", NULL,
+    N_("Show _Status Bar"), NULL, NULL, G_CALLBACK(toggle_pref_cb), FALSE },
+  { "show-toolbar", NULL,
+    N_("Show _Toolbar"), NULL, NULL, G_CALLBACK(toggle_pref_cb), FALSE }
 };
 
 static GtkActionEntry entries[] =
 {
   { "torrent-menu", NULL, N_("_Torrent"), NULL, NULL, NULL },
+  { "view-menu", NULL, N_("_View"), NULL, NULL, NULL },
+  { "sort-menu", NULL, N_("_Sort Torrents By"), NULL, NULL, NULL },
   { "edit-menu", NULL, N_("_Edit"), NULL, NULL, NULL },
   { "help-menu", NULL, N_("_Help"), NULL, NULL, NULL },
   { "priority-menu", NULL, N_("_Priority"), NULL, NULL, NULL },
-  { "add-torrent", GTK_STOCK_OPEN, NULL, NULL, N_("Open Torrent"), G_CALLBACK(action_cb) },
+  { "add-torrent", GTK_STOCK_OPEN,
+    NULL, NULL, N_("Open Torrent"), G_CALLBACK(action_cb) },
   { "start-torrent", GTK_STOCK_MEDIA_PLAY,
     N_("_Start"), "<control>S", NULL, G_CALLBACK(action_cb) },
+  { "show-stats", NULL, N_("_Statistics"), NULL, NULL, G_CALLBACK(action_cb) },
   { "verify-torrent", NULL,
-    N_("_Verify Local Files"), NULL, NULL, G_CALLBACK(action_cb) },
+    N_("_Verify Local Data"), NULL, NULL, G_CALLBACK(action_cb) },
   { "pause-torrent", GTK_STOCK_MEDIA_PAUSE,
     N_("_Pause"), "<control>P", NULL, G_CALLBACK(action_cb) },
   { "remove-torrent", GTK_STOCK_REMOVE,
-    N_("_Remove"), "<control>R", NULL, G_CALLBACK(action_cb) },
+    N_("_Remove"), "<control>Delete", NULL, G_CALLBACK(action_cb) },
   { "create-torrent", GTK_STOCK_NEW,
     N_("Create _New Torrent"), NULL, NULL, G_CALLBACK(action_cb) },
   { "close", GTK_STOCK_CLOSE,
@@ -118,37 +171,40 @@ typedef struct
 }
 BuiltinIconInfo;
 
-/* only one icon now... but room to grow ;) */
-const BuiltinIconInfo my_builtin_icons [] =
+const BuiltinIconInfo my_fallback_icons [] =
 {
-    { tr_icon_logo, "transmission-logo" },
+    { tr_icon_logo, "transmission" },
     { tr_icon_lock, "transmission-lock" }
 };
 
 static void
 register_my_icons ( void )
 {
-   int i;
-   const int n = G_N_ELEMENTS( my_builtin_icons );
-   GtkIconFactory * factory = gtk_icon_factory_new ();
-   gtk_icon_factory_add_default( factory );
+    int i;
+    const int n = G_N_ELEMENTS( my_fallback_icons );
+    GtkIconFactory * factory = gtk_icon_factory_new( );
+    GtkIconTheme * theme = gtk_icon_theme_get_default( );
+    gtk_icon_factory_add_default( factory );
 
-   for( i=0; i<n; ++i )
-   {
-       GdkPixbuf * p;
-       int width;
-       GtkIconSet * icon_set;
+    for( i=0; i<n; ++i )
+    {
+        const char * name = my_fallback_icons[i].name;
 
-       p = gdk_pixbuf_new_from_inline( -1, my_builtin_icons[i].raw, FALSE, 0 );
-       width = gdk_pixbuf_get_width( p );
-       gtk_icon_theme_add_builtin_icon (my_builtin_icons[i].name, width, p );
+        if( !gtk_icon_theme_has_icon( theme, name ) )
+        {
+            int width;
+            GdkPixbuf * p;
+            GtkIconSet * icon_set;
 
-       icon_set = gtk_icon_set_new_from_pixbuf( p );
-       gtk_icon_factory_add( factory, my_builtin_icons[i].name, icon_set );
+            p = gdk_pixbuf_new_from_inline( -1, my_fallback_icons[i].raw, FALSE, 0 );
+            width = gdk_pixbuf_get_width( p );
+            icon_set = gtk_icon_set_new_from_pixbuf( p );
+            gtk_icon_theme_add_builtin_icon( name, width, p );
+            gtk_icon_factory_add( factory, name, icon_set );
 
-       g_object_unref( p );
-       gdk_pixbuf_unref( p );
-       gtk_icon_set_unref (icon_set);
+            g_object_unref( p );
+            gtk_icon_set_unref( icon_set );
+        }
     }
 
     g_object_unref (G_OBJECT (factory));
@@ -156,10 +212,18 @@ register_my_icons ( void )
 
 static GtkUIManager * myUIManager = NULL;
 
+void
+actions_set_core( TrCore * core )
+{
+    myCore = core;
+}
+
 void
 actions_init( GtkUIManager * ui_manager, gpointer callback_user_data )
 {
-  int i;
+  int i, n;
+  int active;
+  char * match;
   const int n_entries = G_N_ELEMENTS( entries );
   GtkActionGroup * action_group;
 
@@ -179,10 +243,32 @@ actions_init( GtkUIManager * ui_manager, gpointer callback_user_data )
                                       TR_PRI_NORMAL,
                                       G_CALLBACK(priority_changed_cb), NULL);
 
-  gtk_action_group_add_toggle_actions ( action_group, 
-					show_toggle_entries, 
-					G_N_ELEMENTS(show_toggle_entries), 
-					callback_user_data );
+
+  match = pref_string_get( PREF_KEY_SORT_MODE );
+  for( i=0, n=G_N_ELEMENTS(sort_radio_entries), active=-1; active==-1 && i<n; ++i )
+      if( !strcmp( sort_radio_entries[i].name, match ) )
+          active = i;
+
+  gtk_action_group_add_radio_actions( action_group,
+                                      sort_radio_entries,
+                                      G_N_ELEMENTS(sort_radio_entries),
+                                      active,
+                                      G_CALLBACK(sort_changed_cb),
+                                      NULL );
+
+  gtk_action_group_add_toggle_actions( action_group, 
+                                       show_toggle_entries, 
+                                       G_N_ELEMENTS(show_toggle_entries), 
+                                       callback_user_data );
+
+  for( i=0, n=G_N_ELEMENTS(pref_toggle_entries); i<n; ++i )
+    pref_toggle_entries[i].is_active =
+      pref_flag_get( pref_toggle_entries[i].name );
+
+  gtk_action_group_add_toggle_actions( action_group, 
+                                       pref_toggle_entries, 
+                                       G_N_ELEMENTS(pref_toggle_entries), 
+                                       callback_user_data );
 
   gtk_action_group_add_actions( action_group,
                                 entries, n_entries,
@@ -190,6 +276,7 @@ actions_init( GtkUIManager * ui_manager, gpointer callback_user_data )
 
   gtk_ui_manager_insert_action_group( ui_manager, action_group, 0 );
   g_object_unref (G_OBJECT(action_group));
+  g_free( match );
 }
 
 /****
