@@ -1,7 +1,7 @@
 /******************************************************************************
- * $Id: conf.h 3381 2007-10-12 20:50:03Z charles $
+ * $Id: conf.h 4404 2008-01-01 17:20:20Z charles $
  *
- * Copyright (c) 2005-2007 Transmission authors and contributors
+ * Copyright (c) 2005-2008 Transmission authors and contributors
  *
  * Permission is hereby granted, free of charge, to any person obtaining a
  * copy of this software and associated documentation files (the "Software"),
@@ -55,5 +55,7 @@ gboolean
 cf_lock(char **errstr);
 char *
 cf_sockname(void);
+void
+cf_check_older_configs(void);
 
 #endif /* TG_CONF_H */
