@@ -1,7 +1,7 @@
 /******************************************************************************
- * $Id: ipc.c 3401 2007-10-13 17:33:53Z charles $
+ * $Id: ipc.c 4404 2008-01-01 17:20:20Z charles $
  *
- * Copyright (c) 2006-2007 Transmission authors and contributors
+ * Copyright (c) 2006-2008 Transmission authors and contributors
  *
  * Permission is hereby granted, free of charge, to any person obtaining a
  * copy of this software and associated documentation files (the "Software"),
@@ -971,6 +971,9 @@ smsg_torall( enum ipc_msg id, benc_val_t * val SHUTUP, int64_t tag,
     simpleresp( con, tag, IPC_MSG_OK );
 }
 
+#define TR_NAT_TRAVERSAL_IS_DISABLED( st ) \
+    ( TR_NAT_TRAVERSAL_UNMAPPED == (st) || TR_NAT_TRAVERSAL_UNMAPPING == (st) )
+
 static void
 smsg_pref( enum ipc_msg id, benc_val_t * val SHUTUP, int64_t tag, void * arg )
 {
