@@ -1,7 +1,7 @@
 /******************************************************************************
- * $Id: torrent-inspector.c 3470 2007-10-19 23:44:01Z charles $
+ * $Id: torrent-inspector.c 4925 2008-02-05 17:14:36Z charles $
  *
- * Copyright (c) 2005-2007 Transmission authors and contributors
+ * Copyright (c) 2005-2008 Transmission authors and contributors
  *
  * Permission is hereby granted, free of charge, to any person obtaining a
  * copy of this software and associated documentation files (the "Software"),
@@ -36,7 +36,7 @@
 #include "torrent-inspector.h"
 #include "util.h"
 
-#define UPDATE_INTERVAL_MSEC 1500
+#define UPDATE_INTERVAL_MSEC 2000
 
 /****
 *****  PIECES VIEW
@@ -231,28 +231,24 @@ refresh_pieces (GtkWidget * da, GdkEventExpose * event UNUSED, gpointer gtor)
 enum
 {
   PEER_COL_ADDRESS,
-  PEER_COL_PORT,
   PEER_COL_CLIENT,
   PEER_COL_PROGRESS,
   PEER_COL_IS_ENCRYPTED,
-  PEER_COL_IS_DOWNLOADING,
   PEER_COL_DOWNLOAD_RATE,
-  PEER_COL_IS_UPLOADING,
   PEER_COL_UPLOAD_RATE,
+  PEER_COL_STATUS,
   N_PEER_COLS
 };
 
 static const char* peer_column_names[N_PEER_COLS] =
 {
   N_("Address"),
-  N_("Port"),
   N_("Client"),
-  N_("Progress"),
+  N_("%"),
   " ",
-  N_("Downloading"),
-  N_("DL Rate"),
-  N_("Uploading"),
-  N_("UL Rate")
+  N_("Down"),
+  N_("Up"),
+  N_("Status")
 };
 
 static int compare_peers (const void * a, const void * b)
@@ -280,14 +276,12 @@ peer_row_set (GtkTreeStore        * store,
 
   gtk_tree_store_set (store, iter,
                       PEER_COL_ADDRESS, peer->addr,
-                      PEER_COL_PORT, peer->port,
                       PEER_COL_CLIENT, client,
                       PEER_COL_IS_ENCRYPTED, peer->isEncrypted,
-                      PEER_COL_PROGRESS, (int)(100.0*peer->progress + 0.5),
-                      PEER_COL_IS_DOWNLOADING, peer->isDownloading,
+                      PEER_COL_PROGRESS, (int)(100.0*peer->progress),
                       PEER_COL_DOWNLOAD_RATE, peer->downloadFromRate,
-                      PEER_COL_IS_UPLOADING, peer->isUploading,
                       PEER_COL_UPLOAD_RATE, peer->uploadToRate,
+                      PEER_COL_STATUS, peer->status,
                       -1);
 }
 
@@ -309,14 +303,12 @@ peer_model_new (tr_torrent * tor)
 {
   GtkTreeStore * m = gtk_tree_store_new (N_PEER_COLS,
                                          G_TYPE_STRING,  /* addr */
-                                         G_TYPE_INT,     /* port */
                                          G_TYPE_STRING,  /* client */
                                          G_TYPE_INT,     /* progress [0..100] */
                                          G_TYPE_BOOLEAN, /* isEncrypted */
-                                         G_TYPE_BOOLEAN, /* isDownloading */
                                          G_TYPE_FLOAT,   /* downloadFromRate */
-                                         G_TYPE_BOOLEAN, /* isUploading */
-                                         G_TYPE_FLOAT);  /* uploadToRate */
+                                         G_TYPE_FLOAT,   /* uploadToRate */
+                                         G_TYPE_INT );   /* tr_peer_status */
 
   int n_peers = 0;
   tr_peer_stat * peers = tr_torrentPeers (tor, &n_peers);
@@ -342,6 +334,46 @@ render_encrypted (GtkTreeViewColumn  * column UNUSED,
                           NULL);
 }
 
+static void
+render_status( GtkTreeViewColumn  * column UNUSED,
+               GtkCellRenderer    * renderer,
+               GtkTreeModel       * tree_model,
+               GtkTreeIter        * iter,
+               gpointer             data UNUSED )
+{
+    GString * gstr = g_string_new( NULL );
+    int status;
+    gtk_tree_model_get( tree_model, iter, PEER_COL_STATUS, &status, -1 );
+
+    if( status & TR_PEER_STATUS_HANDSHAKE )
+    {
+        g_string_append( gstr, _("Handshaking") );
+    }
+    else
+    {
+        if( status & TR_PEER_STATUS_CLIENT_IS_SENDING )
+            g_string_append( gstr, _("Uploading to peer") );
+        else if( status & TR_PEER_STATUS_PEER_IS_INTERESTED )
+            g_string_append( gstr, _("Peer wants our data") );
+        else if( status & TR_PEER_STATUS_PEER_IS_CHOKED )
+            g_string_append( gstr, _("Refusing to send data to peer") );
+
+        g_string_append( gstr, " - " );
+
+        if( status & TR_PEER_STATUS_PEER_IS_SENDING )
+            g_string_append( gstr, _("Downloading from peer") );
+        else if( status & TR_PEER_STATUS_CLIENT_SENT_REQUEST )
+            g_string_append( gstr, _("Requesting data from peer") );
+        else if( status & TR_PEER_STATUS_CLIENT_IS_INTERESTED )
+            g_string_append( gstr, _("Waiting to request data from peer") );
+        else if( status & TR_PEER_STATUS_CLIENT_IS_CHOKED )
+            g_string_append( gstr, _("Peer will not send us data") );
+    }
+
+    g_object_set( renderer, "text", gstr->str, NULL );
+    g_string_free( gstr, TRUE );
+}
+
 static void
 render_ul_rate (GtkTreeViewColumn  * column UNUSED,
                 GtkCellRenderer    * renderer,
@@ -354,9 +386,9 @@ render_ul_rate (GtkTreeViewColumn  * column UNUSED,
   if( rate < 0.01 )
     g_object_set (renderer, "text", "", NULL);
   else {
-    char * pch = readablespeed (rate);
-    g_object_set (renderer, "text", pch, NULL);
-    g_free (pch); 
+    char speedStr[64];
+    tr_strlspeed( speedStr, rate, sizeof(speedStr) );
+    g_object_set( renderer, "text", speedStr, NULL );
   }
 }
 
@@ -372,9 +404,9 @@ render_dl_rate (GtkTreeViewColumn  * column UNUSED,
   if( rate < 0.01 )
     g_object_set (renderer, "text", "", NULL);
   else {
-    char * pch = readablespeed (rate);
-    g_object_set (renderer, "text", pch, NULL);
-    g_free (pch); 
+    char speedStr[64];
+    tr_strlspeed( speedStr, rate, sizeof(speedStr) );
+    g_object_set( renderer, "text", speedStr, NULL );
   }
 }
 
@@ -477,7 +509,7 @@ refresh_peers (GtkWidget * top)
   fmtpeercount (p->leechers_lb, stat->leechers);
   fmtpeercount (p->completed_lb, stat->completedFromTracker );
 
-  free (peers);
+  free( peers );
 }
 
 static GtkWidget* peer_page_new ( TrTorrent * gtor )
@@ -490,12 +522,16 @@ static GtkWidget* peer_page_new ( TrTorrent * gtor )
   char name[64];
 
   /* TODO: make this configurable? */
-  int view_columns[] = { PEER_COL_ADDRESS,
-                         PEER_COL_CLIENT,
-                         PEER_COL_PROGRESS,
-                         PEER_COL_IS_ENCRYPTED,
+  int view_columns[] = { PEER_COL_IS_ENCRYPTED,
                          PEER_COL_UPLOAD_RATE,
-                         PEER_COL_DOWNLOAD_RATE };
+                         PEER_COL_DOWNLOAD_RATE,
+                         PEER_COL_PROGRESS,
+                         PEER_COL_ADDRESS,
+                         PEER_COL_CLIENT
+#if 0
+                         , PEER_COL_STATUS
+#endif
+                       };
 
   m  = peer_model_new (tor);
   v = gtk_tree_view_new_with_model (m);
@@ -514,25 +550,21 @@ static GtkWidget* peer_page_new ( TrTorrent * gtor )
     {
       case PEER_COL_ADDRESS:
         r = gtk_cell_renderer_text_new ();
-        c = gtk_tree_view_column_new_with_attributes (t, r, "text", col, NULL);
-        break;
-
-      case PEER_COL_PORT:
-        r = gtk_cell_renderer_text_new ();
+        g_object_set( G_OBJECT( r ), "ellipsize", PANGO_ELLIPSIZE_END, NULL );
         c = gtk_tree_view_column_new_with_attributes (t, r, "text", col, NULL);
         break;
 
       case PEER_COL_CLIENT:
         r = gtk_cell_renderer_text_new ();
-        c = gtk_tree_view_column_new_with_attributes (t, r, "text", col, NULL);
+        g_object_set( G_OBJECT( r ), "ellipsize", PANGO_ELLIPSIZE_END, NULL );
+        c = gtk_tree_view_column_new_with_attributes (t, r, "text", col, NULL );
         gtk_tree_view_column_set_cell_data_func (c, r, render_client,
                                                  NULL, NULL);
         break;
 
       case PEER_COL_PROGRESS:
         r = gtk_cell_renderer_progress_new ();
-        c = gtk_tree_view_column_new_with_attributes (
-              _("Progress"), r, "value", PEER_COL_PROGRESS, NULL);
+        c = gtk_tree_view_column_new_with_attributes (t, r, "value", PEER_COL_PROGRESS, NULL);
         break;
 
       case PEER_COL_IS_ENCRYPTED:
@@ -545,11 +577,6 @@ static GtkWidget* peer_page_new ( TrTorrent * gtor )
                                                  NULL, NULL);
         break;
 
-      case PEER_COL_IS_DOWNLOADING:
-        r = gtk_cell_renderer_text_new ();
-        c = gtk_tree_view_column_new_with_attributes (t, r, "text", col, NULL);
-        break;
-
       case PEER_COL_DOWNLOAD_RATE:
         r = gtk_cell_renderer_text_new ();
         c = gtk_tree_view_column_new_with_attributes (t, r, "text", col, NULL);
@@ -557,11 +584,6 @@ static GtkWidget* peer_page_new ( TrTorrent * gtor )
                                                  NULL, NULL);
         break;
 
-      case PEER_COL_IS_UPLOADING:
-        r = gtk_cell_renderer_text_new ();
-        c = gtk_tree_view_column_new_with_attributes (t, r, "text", col, NULL);
-        break;
-
       case PEER_COL_UPLOAD_RATE:
         r = gtk_cell_renderer_text_new ();
         c = gtk_tree_view_column_new_with_attributes (t, r, "text", col, NULL);
@@ -569,6 +591,13 @@ static GtkWidget* peer_page_new ( TrTorrent * gtor )
                                                  NULL, NULL);
         break;
 
+      case PEER_COL_STATUS:
+        r = gtk_cell_renderer_text_new( );
+        c = gtk_tree_view_column_new_with_attributes (t, r, "text", col, NULL);
+        gtk_tree_view_column_set_cell_data_func (c, r, render_status, NULL, NULL);
+        break;
+        
+
       default:
         abort ();
     }
@@ -608,23 +637,23 @@ static GtkWidget* peer_page_new ( TrTorrent * gtor )
     gtk_box_pack_start (GTK_BOX(vbox), l, FALSE, FALSE, 0);
 
     w = da = p->completeness = gtk_drawing_area_new ();
-    gtk_widget_set_usize (w, 0u, 100u);
+    gtk_widget_set_size_request (w, 0u, 100u);
     g_object_set_data (G_OBJECT(w), "draw-mode", GINT_TO_POINTER(DRAW_AVAIL));
     g_signal_connect (w, "expose-event", G_CALLBACK(refresh_pieces), gtor);
 
     h = gtk_hbox_new (FALSE, GUI_PAD);
     w = gtk_alignment_new (0.0f, 0.0f, 0.0f, 0.0f);
-    gtk_widget_set_usize (w, GUI_PAD_BIG, 0);
+    gtk_widget_set_size_request (w, GUI_PAD_BIG, 0);
     gtk_box_pack_start (GTK_BOX(h), w, FALSE, FALSE, 0);
     gtk_box_pack_start_defaults (GTK_BOX(h), da);
     gtk_box_pack_start (GTK_BOX(vbox), h, FALSE, FALSE, 0);
 
     /* a small vertical spacer */
     w = gtk_alignment_new (0.0f, 0.0f, 0.0f, 0.0f);
-    gtk_widget_set_usize (w, 0u, GUI_PAD);
+    gtk_widget_set_size_request (w, 0u, GUI_PAD);
     gtk_box_pack_start (GTK_BOX(vbox), w, FALSE, FALSE, 0);
 
-    g_snprintf (name, sizeof(name), "<b>%s</b>", _("Peers"));
+    g_snprintf (name, sizeof(name), "<b>%s</b>", _("Connected Peers"));
     l = gtk_label_new (NULL);
     gtk_misc_set_alignment (GTK_MISC(l), 0.0f, 0.5f);
     gtk_label_set_markup (GTK_LABEL(l), name);
@@ -632,14 +661,14 @@ static GtkWidget* peer_page_new ( TrTorrent * gtor )
 
     h = gtk_hbox_new (FALSE, GUI_PAD);
     w = gtk_alignment_new (0.0f, 0.0f, 0.0f, 0.0f);
-    gtk_widget_set_usize (w, GUI_PAD_BIG, 0);
+    gtk_widget_set_size_request (w, GUI_PAD_BIG, 0);
     gtk_box_pack_start (GTK_BOX(h), w, FALSE, FALSE, 0);
     gtk_box_pack_start_defaults (GTK_BOX(h), sw);
     gtk_box_pack_start_defaults (GTK_BOX(vbox), h);
 
     hbox = gtk_hbox_new (FALSE, GUI_PAD);
     w = gtk_alignment_new (0.0f, 0.0f, 0.0f, 0.0f);
-    gtk_widget_set_usize (w, GUI_PAD_BIG, 0);
+    gtk_widget_set_size_request (w, GUI_PAD_BIG, 0);
     gtk_box_pack_start (GTK_BOX(hbox), w, FALSE, FALSE, 0);
         g_snprintf (name, sizeof(name), "<b>%s:</b>", _("Seeders"));
         l = gtk_label_new (NULL);
@@ -684,6 +713,7 @@ static GtkWidget* info_page_new (tr_torrent * tor)
   GtkWidget *l, *w, *fr;
   char *pch;
   char *dname, *bname;
+  char sizeStr[128];
   char buf[256];
   char name[128];
   const char * namefmt = "%s:";
@@ -697,21 +727,23 @@ static GtkWidget* info_page_new (tr_torrent * tor)
     g_snprintf (name, sizeof(name), namefmt, _("Tracker"));
     track = info->trackerList->list;
     pch = track->port==80
-      ? g_strdup_printf ("http://%s%s", track->address, track->announce)
-      : g_strdup_printf ("http://%s:%d%s", track->address, track->port, track->announce);
-    l = gtk_label_new (pch);
+      ? g_strdup_printf( "http://%s%s", track->address, track->announce )
+      : g_strdup_printf( "http://%s:%d%s", track->address, track->port, track->announce );
+    l = gtk_label_new( pch );
+    gtk_label_set_ellipsize( GTK_LABEL( l ), PANGO_ELLIPSIZE_END );
+
     hig_workarea_add_row (t, &row, name, l, NULL);
     g_free (pch);
 
     g_snprintf (name, sizeof(name), namefmt, _("Pieces"));
-    pch = readablesize (info->pieceSize);
-    g_snprintf (buf, sizeof(buf), "%d (%s)", info->pieceCount, pch);
+    tr_strlsize( sizeStr, info->pieceSize, sizeof(sizeStr) );
+    g_snprintf( buf, sizeof(buf), "%d (%s)", info->pieceCount, sizeStr );
     l = gtk_label_new (buf);
     hig_workarea_add_row (t, &row, name, l, NULL);
-    g_free (pch);
 
     g_snprintf (name, sizeof(name), namefmt, _("Hash"));
     l = gtk_label_new (info->hashString);
+    gtk_label_set_ellipsize( GTK_LABEL( l ), PANGO_ELLIPSIZE_END );
     hig_workarea_add_row (t, &row, name, l, NULL);
 
     g_snprintf (name, sizeof(name), namefmt, _("Secure"));
@@ -753,17 +785,20 @@ static GtkWidget* info_page_new (tr_torrent * tor)
 
     g_snprintf (name, sizeof(name), namefmt, _("Downloaded Data"));
     l = gtk_label_new (tr_torrentGetFolder (tor));
+    gtk_label_set_ellipsize( GTK_LABEL( l ), PANGO_ELLIPSIZE_END );
     hig_workarea_add_row (t, &row, name, l, NULL); 
 
     g_snprintf (name, sizeof(name), namefmt, _("Torrent File Path"));
     dname = g_path_get_dirname (info->torrent);
     l = gtk_label_new ( dname );
+    gtk_label_set_ellipsize( GTK_LABEL( l ), PANGO_ELLIPSIZE_END );
     hig_workarea_add_row (t, &row, name, l, NULL); 
     g_free (dname);
 
     g_snprintf (name, sizeof(name), namefmt, _("Torrent File Name"));
     bname = g_path_get_basename (info->torrent);
     l = gtk_label_new (bname);
+    gtk_label_set_ellipsize( GTK_LABEL( l ), PANGO_ELLIPSIZE_END );
     hig_workarea_add_row (t, &row, name, l, NULL); 
     g_free (bname);
 
@@ -778,13 +813,13 @@ static GtkWidget* info_page_new (tr_torrent * tor)
 typedef struct
 {
   GtkWidget * state_lb;
-  GtkWidget * corrupt_dl_lb;
-  GtkWidget * valid_dl_lb;
+  GtkWidget * progress_lb;
+  GtkWidget * have_lb;
   GtkWidget * dl_lb;
   GtkWidget * ul_lb;
+  GtkWidget * failed_lb;
   GtkWidget * ratio_lb;
   GtkWidget * err_lb;
-  GtkWidget * remaining_lb;
   GtkWidget * swarm_lb;
   GtkWidget * date_added_lb;
   GtkWidget * last_activity_lb;
@@ -799,38 +834,37 @@ refresh_activity (GtkWidget * top)
   Activity * a = (Activity*) g_object_get_data (G_OBJECT(top), "activity-data");
   const tr_stat * stat = tr_torrent_stat( a->gtor );
   char *pch;
+  char sizeStr[64];
+  char sizeStr2[64];
+  char buf[128];
 
   pch = tr_torrent_status_str( a->gtor );
   gtk_label_set_text (GTK_LABEL(a->state_lb), pch);
   g_free (pch);
 
-  pch = readablesize (stat->corruptEver);
-  gtk_label_set_text (GTK_LABEL(a->corrupt_dl_lb), pch);
+  pch = g_strdup_printf( _( "%.1f%% (%.1f%% selected)" ), stat->percentComplete*100.0, stat->percentDone*100.0 );
+  gtk_label_set_text (GTK_LABEL(a->progress_lb), pch);
   g_free (pch);
 
-  pch = readablesize (stat->haveValid);
-  gtk_label_set_text (GTK_LABEL(a->valid_dl_lb), pch);
-  g_free (pch);
+  tr_strlsize( sizeStr,  stat->haveValid + stat->haveUnchecked, sizeof(sizeStr) );
+  tr_strlsize( sizeStr2, stat->haveValid,                       sizeof(sizeStr2) );
+  g_snprintf( buf, sizeof(buf), _("%s (%s verified)"), sizeStr, sizeStr2 );
+  gtk_label_set_text( GTK_LABEL( a->have_lb ), buf );
 
-  pch = readablesize (stat->downloadedEver);
-  gtk_label_set_text (GTK_LABEL(a->dl_lb), pch);
-  g_free (pch);
+  tr_strlsize( sizeStr, stat->downloadedEver, sizeof(sizeStr) );
+  gtk_label_set_text( GTK_LABEL(a->dl_lb), sizeStr );
 
-  pch = readablesize (stat->uploadedEver);
-  gtk_label_set_text (GTK_LABEL(a->ul_lb), pch);
-  g_free (pch);
+  tr_strlsize( sizeStr, stat->uploadedEver, sizeof(sizeStr) );
+  gtk_label_set_text( GTK_LABEL(a->ul_lb), sizeStr );
 
-  pch = ratiostr (stat->downloadedEver, stat->uploadedEver);
-  gtk_label_set_text (GTK_LABEL(a->ratio_lb), pch);
-  g_free (pch);
+  tr_strlsize( sizeStr, stat->corruptEver, sizeof(sizeStr) );
+  gtk_label_set_text (GTK_LABEL(a->failed_lb), sizeStr );
 
-  pch = readablespeed (stat->swarmspeed);
-  gtk_label_set_text (GTK_LABEL(a->swarm_lb), pch);
-  g_free (pch);
+  g_snprintf( buf, sizeof(buf), "%.1f", stat->ratio );
+  gtk_label_set_text (GTK_LABEL(a->ratio_lb), buf );
 
-  pch = readablesize (stat->leftUntilDone);
-  gtk_label_set_text (GTK_LABEL(a->remaining_lb), pch);
-  g_free (pch);
+  tr_strlspeed( buf, stat->swarmspeed, sizeof(buf) );
+  gtk_label_set_text (GTK_LABEL(a->swarm_lb), buf );
 
   gtk_label_set_text (GTK_LABEL(a->err_lb),
                       *stat->errorString ? stat->errorString : _("None"));
@@ -857,68 +891,54 @@ activity_page_new (TrTorrent * gtor)
   int row = 0;
   GtkWidget *t = hig_workarea_create ();
   GtkWidget *l, *w;
-  char name[128];
-  const char * namefmt = "%s:";
 
   a->gtor = gtor;
 
   hig_workarea_add_section_title (t, &row, _("Transfer"));
   hig_workarea_add_section_spacer (t, row, 8);
 
-    g_snprintf (name, sizeof(name), namefmt, _("State"));
     l = a->state_lb = gtk_label_new (NULL);
-    hig_workarea_add_row (t, &row, name, l, NULL);
+    hig_workarea_add_row (t, &row, _("State:"), l, NULL);
 
-    g_snprintf (name, sizeof(name), namefmt, _("Corrupt DL"));
-    l = a->corrupt_dl_lb = gtk_label_new (NULL);
-    hig_workarea_add_row (t, &row, name, l, NULL);
+    l = a->progress_lb = gtk_label_new (NULL);
+    hig_workarea_add_row (t, &row, _("Progress:"), l, NULL);
 
-    g_snprintf (name, sizeof(name), namefmt, _("Valid DL"));
-    l = a->valid_dl_lb = gtk_label_new (NULL);
-    hig_workarea_add_row (t, &row, name, l, NULL);
+    l = a->have_lb = gtk_label_new (NULL);
+    hig_workarea_add_row (t, &row, _("Have:"), l, NULL);
 
-    g_snprintf (name, sizeof(name), namefmt, _("Downloaded"));
     l = a->dl_lb = gtk_label_new (NULL);
-    hig_workarea_add_row (t, &row, name, l, NULL);
+    hig_workarea_add_row (t, &row, _("Downloaded:"), l, NULL);
 
-    g_snprintf (name, sizeof(name), namefmt, _("Uploaded"));
     l = a->ul_lb = gtk_label_new (NULL);
-    hig_workarea_add_row (t, &row, name, l, NULL);
+    hig_workarea_add_row (t, &row, _("Uploaded:"), l, NULL);
+
+    l = a->failed_lb = gtk_label_new (NULL);
+    hig_workarea_add_row (t, &row, _("Failed DL:"), l, NULL);
 
-    g_snprintf (name, sizeof(name), namefmt, _("Ratio"));
     l = a->ratio_lb = gtk_label_new (NULL);
-    hig_workarea_add_row (t, &row, name, l, NULL);
+    hig_workarea_add_row (t, &row, _("Ratio:"), l, NULL);
 
-    g_snprintf (name, sizeof(name), namefmt, _("Remaining"));
-    l = a->remaining_lb = gtk_label_new (NULL);
-    hig_workarea_add_row (t, &row, name, l, NULL);
-
-    g_snprintf (name, sizeof(name), namefmt, _("Swarm Rate"));
     l = a->swarm_lb = gtk_label_new (NULL);
-    hig_workarea_add_row (t, &row, name, l, NULL);
+    hig_workarea_add_row (t, &row, _("Swarm Rate:"), l, NULL);
 
-    g_snprintf (name, sizeof(name), namefmt, _("Error"));
     l = a->err_lb = gtk_label_new (NULL);
-    hig_workarea_add_row (t, &row, name, l, NULL);
+    hig_workarea_add_row (t, &row, _("Error:"), l, NULL);
 
-    g_snprintf (name, sizeof(name), namefmt, _("Completeness"));
     w = a->availability_da = gtk_drawing_area_new ();
-    gtk_widget_set_usize (w, 0u, 100u);
+    gtk_widget_set_size_request (w, 0u, 100u);
     g_object_set_data (G_OBJECT(w), "draw-mode", GINT_TO_POINTER(DRAW_PROG));
     g_signal_connect (w, "expose-event", G_CALLBACK(refresh_pieces), gtor);
-    hig_workarea_add_row (t, &row, name, w, NULL);
+    hig_workarea_add_row (t, &row, _("Completeness:"), w, NULL);
 
   hig_workarea_add_section_divider (t, &row);
   hig_workarea_add_section_title (t, &row, _("Dates"));
   hig_workarea_add_section_spacer (t, row, 3);
 
-    g_snprintf (name, sizeof(name), namefmt, _("Added"));
     l = a->date_added_lb = gtk_label_new (NULL);
-    hig_workarea_add_row (t, &row, name, l, NULL);
+    hig_workarea_add_row (t, &row, _("Added:"), l, NULL);
 
-    g_snprintf (name, sizeof(name), namefmt, _("Last Activity"));
     l = a->last_activity_lb = gtk_label_new (NULL);
-    hig_workarea_add_row (t, &row, name, l, NULL);
+    hig_workarea_add_row (t, &row, _("Last Activity"), l, NULL);
 
   hig_workarea_add_section_divider (t, &row);
   hig_workarea_finish (t, &row);
@@ -1053,12 +1073,13 @@ getdirtotals( GtkTreeStore * store, GtkTreeIter * parent )
     GtkTreeModel * model;
     GtkTreeIter    iter;
     uint64_t       mysize, subsize;
-    char         * sizestr, * name, * label;
+    char         * name, * label;
 
     model  = GTK_TREE_MODEL( store );
     mysize = 0;
     if( gtk_tree_model_iter_children( model, &iter, parent ) ) do
     {
+         char sizeStr[64];
         if( gtk_tree_model_iter_has_child( model, &iter ) )
         {
             subsize = getdirtotals( store, &iter );
@@ -1069,10 +1090,9 @@ getdirtotals( GtkTreeStore * store, GtkTreeIter * parent )
             gtk_tree_model_get( model, &iter, FC_SIZE, &subsize, -1 );
         }
         gtk_tree_model_get( model, &iter, FC_LABEL, &name, -1 );
-        sizestr = readablesize( subsize );
+        tr_strlsize( sizeStr, subsize, sizeof( sizeStr ) );
         label = g_markup_printf_escaped( "<small>%s (%s)</small>",
-                                          name, sizestr );
-        g_free( sizestr );
+                                          name, sizeStr );
         g_free( name );
         gtk_tree_store_set( store, &iter, FC_LABEL, label, -1 );
         g_free( label );
@@ -1110,7 +1130,7 @@ updateprogress( GtkTreeModel   * model,
                                               FC_INDEX, &index,
                                               -1 );
             g_assert( 0 <= index );
-            percent = (int)(fileStats[index].progress * 100.0 + 0.5); /* [0...100] */
+            percent = (int)(fileStats[index].progress * 100.0); /* [0...100] */
             subGot = (guint64)(subTotal * percent/100.0);
         }
 
@@ -1120,10 +1140,10 @@ updateprogress( GtkTreeModel   * model,
         /* why not just set it every time?
            because that causes the "priorities" combobox to pop down */
         gtk_tree_model_get (model, &iter, FC_PROG, &oldProg, -1);
-        newProg = (int)(100.0*subGot/subTotal + 0.5);
+        newProg = (int)(100.0*subGot/subTotal);
         if (oldProg != newProg)
           gtk_tree_store_set (store, &iter,
-                              FC_PROG, (int)(100.0*subGot/subTotal + 0.5), -1);
+                              FC_PROG, (int)(100.0*subGot/subTotal), -1);
 
         gotSize += subGot;
         totalSize += subTotal;
@@ -1321,7 +1341,7 @@ enabled_toggled (GtkCellRendererToggle  * cell UNUSED,
   gtk_tree_path_free( path );
 }
 
-GtkWidget *
+static GtkWidget *
 file_page_new ( TrTorrent * gtor )
 {
     GtkWidget           * ret;
@@ -1369,7 +1389,6 @@ file_page_new ( TrTorrent * gtor )
     /* add file column */
     
     col = GTK_TREE_VIEW_COLUMN (g_object_new (GTK_TYPE_TREE_VIEW_COLUMN,
-        "sizing", GTK_TREE_VIEW_COLUMN_AUTOSIZE,
         "expand", TRUE,
         "title", _("File"),
         NULL));
@@ -1428,7 +1447,7 @@ file_page_new ( TrTorrent * gtor )
     gtk_scrolled_window_set_shadow_type (GTK_SCROLLED_WINDOW(scroll),
                                          GTK_SHADOW_IN);
     gtk_container_add( GTK_CONTAINER( scroll ), view );
-    gtk_widget_set_usize (scroll, 0u, 200u);
+    gtk_widget_set_size_request (scroll, 0u, 200u);
     gtk_container_set_border_width (GTK_CONTAINER(scroll), GUI_PAD);
 
     ret = scroll;
@@ -1480,12 +1499,14 @@ dl_speed_toggled_cb (GtkToggleButton *tb, gpointer gtor)
 }
 
 
+#if 0
 static void
 seeding_cap_toggled_cb (GtkToggleButton *tb, gpointer gtor)
 {
   tr_torrent_set_seeding_cap_enabled (TR_TORRENT(gtor), 
                                       gtk_toggle_button_get_active(tb));
 }
+#endif
 
 static void
 sensitize_from_check_cb (GtkToggleButton *toggle, gpointer w)
@@ -1512,6 +1533,7 @@ dl_speed_spun_cb (GtkSpinButton *spin, gpointer gtor)
   setSpeedLimit( spin, gtor, TR_DOWN );
 }
 
+#if 0
 static void
 seeding_ratio_spun_cb (GtkSpinButton *spin, gpointer gtor)
 {
@@ -1519,9 +1541,18 @@ seeding_ratio_spun_cb (GtkSpinButton *spin, gpointer gtor)
                                     gtk_spin_button_get_value(spin));
 }
 
-GtkWidget*
+static void
+max_peers_spun_cb( GtkSpinButton * spin, gpointer gtor )
+{
+  const uint16_t n = gtk_spin_button_get_value( spin );
+  tr_torrentSetMaxConnectedPeers( tr_torrent_handle( gtor ), n );
+}
+#endif
+
+static GtkWidget*
 options_page_new ( TrTorrent * gtor )
 {
+  /* uint16_t maxConnectedPeers; */
   int i, row;
   gboolean b;
   GtkAdjustment *a;
@@ -1559,10 +1590,17 @@ options_page_new ( TrTorrent * gtor )
     sensitize_from_check_cb (GTK_TOGGLE_BUTTON(tb), w);
     hig_workarea_add_row_w (t, &row, tb, w, NULL);
 
+#if 0
   hig_workarea_add_section_divider (t, &row);
-  hig_workarea_add_section_title (t, &row, _("Seeding"));
+  hig_workarea_add_section_title (t, &row, _("Peer Connections"));
   hig_workarea_add_section_spacer (t, row, 1);
 
+    maxConnectedPeers = tr_torrentGetMaxConnectedPeers( tor );
+    w = gtk_spin_button_new_with_range( 1, 3000, 5 );
+    gtk_spin_button_set_value( GTK_SPIN_BUTTON( w ), maxConnectedPeers );
+    hig_workarea_add_row( t, &row, _( "Maximum connected peers:" ), w, NULL );
+    g_signal_connect( w, "value-changed", G_CALLBACK( max_peers_spun_cb ), gtor );
+
     tb = gtk_check_button_new_with_mnemonic (_("_Stop Seeding at Ratio:"));
     gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON(tb), gtor->seeding_cap_enabled);
     g_signal_connect (tb, "toggled", G_CALLBACK(seeding_cap_toggled_cb), gtor);
@@ -1572,6 +1610,7 @@ options_page_new ( TrTorrent * gtor )
     g_signal_connect (tb, "toggled", G_CALLBACK(sensitize_from_check_cb), w);
     sensitize_from_check_cb (GTK_TOGGLE_BUTTON(tb), w);
     hig_workarea_add_row_w (t, &row, tb, w, NULL);
+#endif
 
   hig_workarea_finish (t, &row);
   return t;
@@ -1620,22 +1659,22 @@ GtkWidget*
 torrent_inspector_new ( GtkWindow * parent, TrTorrent * gtor )
 {
   guint tag;
-  char *size, *pch;
   GtkWidget *d, *n, *w;
   tr_torrent * tor = tr_torrent_handle (gtor);
+  char sizeStr[64];
+  char title[512];
   const tr_info * info = tr_torrent_info (gtor);
 
   /* create the dialog */
-  size = readablesize( info->totalSize );
-  pch = g_strdup_printf( _( "Details for %s (%s)" ), info->name, size );
-  d = gtk_dialog_new_with_buttons (pch, parent, 0,
+  tr_strlsize( sizeStr, info->totalSize, sizeof(sizeStr) );
+  g_snprintf( title, sizeof(title), _( "Details for %s (%s)" ), info->name, sizeStr );
+  d = gtk_dialog_new_with_buttons (title, parent, 0,
                                    GTK_STOCK_CLOSE, GTK_RESPONSE_CLOSE,
                                    NULL);
   gtk_window_set_role (GTK_WINDOW(d), "tr-info" );
   g_signal_connect (d, "response", G_CALLBACK (response_cb), gtor);
+  gtk_dialog_set_has_separator( GTK_DIALOG( d ), FALSE );
   g_object_weak_ref (G_OBJECT(gtor), torrent_destroyed, d);
-  g_free( pch );
-  g_free( size );
 
   /* add the notebook */
   n = gtk_notebook_new ();
