@@ -1,7 +1,7 @@
 /******************************************************************************
- * $Id: tr_core.c 3573 2007-10-26 03:02:35Z charles $
+ * $Id: tr_core.c 4977 2008-02-08 03:00:14Z charles $
  *
- * Copyright (c) 2007 Transmission authors and contributors
+ * Copyright (c) 2007-2008 Transmission authors and contributors
  *
  * Permission is hereby granted, free of charge, to any person obtaining a
  * copy of this software and associated documentation files (the "Software"),
@@ -122,39 +122,12 @@ tr_core_dispose( GObject * obj )
 {
     TrCore       * self = (TrCore *) obj;
     GObjectClass * parent;
-    GtkTreeIter    iter;
-    TrTorrent    * tor;
 
     if( self->disposed )
-    {
         return;
-    }
+
     self->disposed = TRUE;
-
     pref_save( NULL );
-
-#ifdef REFDBG
-    fprintf( stderr, "core    %p dispose\n", self );
-#endif
-
-    /* sever all remaining torrents in the model */
-    if( gtk_tree_model_get_iter_first( self->model, &iter ) ) do
-    {
-        gtk_tree_model_get( self->model, &iter, MC_TORRENT, &tor, -1 );
-        tr_torrent_sever( tor );
-        g_object_unref( tor );
-    }
-    while( gtk_tree_model_iter_next( self->model, &iter ) );
-    g_object_unref( self->model );
-
-#ifdef REFDBG
-    fprintf( stderr, "core    %p dead\n", self );
-#endif
-
-    /* close the libtransmission instance */
-    tr_close( self->handle );
-
-    /* Chain up to the parent class */
     parent = g_type_class_peek( g_type_parent( TR_CORE_TYPE ) );
     parent->dispose( obj );
 }
@@ -198,124 +171,203 @@ tr_core_class_init( gpointer g_class, gpointer g_class_data SHUTUP )
 }
 
 static int
-compareProgress( GtkTreeModel   * model,
-                 GtkTreeIter    * a,
-                 GtkTreeIter    * b,
-                 gpointer         user_data UNUSED )
+compareDouble( double a, double b )
 {
-    int ia, ib;
-    gfloat rateUpA, rateUpB;
-    gfloat rateDownA, rateDownB;
-    gfloat percentDoneA, percentDoneB;
-    guint64 uploadedEverA, uploadedEverB;
+    if( a < b ) return -1;
+    if( b < a ) return 1;
+    return 0;
+}
 
-    gtk_tree_model_get( model, a, MC_PROG_D, &percentDoneA,
-                                  MC_DRATE, &rateDownA,
-                                  MC_URATE, &rateUpA,
-                                  MC_UP, &uploadedEverA,
-                                  -1 );
-    gtk_tree_model_get( model, b, MC_PROG_D, &percentDoneB,
-                                  MC_DRATE, &rateDownB,
-                                  MC_URATE, &rateUpB,
-                                  MC_UP, &uploadedEverB,
-                                  -1 );
+static int
+compareByActivity( GtkTreeModel * model,
+                   GtkTreeIter  * a,
+                   GtkTreeIter  * b,
+                   gpointer       user_data UNUSED )
+{
+    int i;
+    tr_torrent *ta, *tb;
+    const tr_stat *sa, *sb;
 
-    ia = (int)( rateUpA + rateDownA );
-    ib = (int)( rateUpB + rateDownB );
-    if( ia != ib )
-        return ia - ib;
+    gtk_tree_model_get( model, a, MC_TORRENT_RAW, &ta, -1 );
+    gtk_tree_model_get( model, b, MC_TORRENT_RAW, &tb, -1 );
 
-    ia = (int)( 100.0 * percentDoneA );
-    ib = (int)( 100.0 * percentDoneB );
-    if( ia != ib )
-        return ia - ib;
+    sa = tr_torrentStatCached( ta );
+    sb = tr_torrentStatCached( tb );
 
-    if( uploadedEverA != uploadedEverB )
-        return uploadedEverA < uploadedEverB ? -1 : 1;
+    if(( i = compareDouble( sa->rateUpload + sa->rateDownload,
+                            sb->rateUpload + sb->rateDownload ) ))
+        return i;
+
+    if( sa->uploadedEver != sb->uploadedEver )
+        return sa->uploadedEver < sa->uploadedEver ? -1 : 1;
 
     return 0;
 }
 
-#define STR_REVERSE "reverse-"
-#define STR_PROGRESS "progress"
-#define STR_NAME "name"
-
-static void
-onSortColumnChanged( GtkTreeSortable * sortable, gpointer unused UNUSED )
+static int
+compareByDateAdded( GtkTreeModel   * model UNUSED,
+                    GtkTreeIter    * a UNUSED,
+                    GtkTreeIter    * b UNUSED,
+                    gpointer         user_data UNUSED )
 {
-    int column;
-    GtkSortType order;
-    if( gtk_tree_sortable_get_sort_column_id( sortable, &column, &order ) )
-    {
-        GString * gstr = g_string_new( NULL );
-        switch( column ) {
-            case MC_PROG_D: g_string_assign( gstr, STR_PROGRESS ); break;
-            default: g_string_assign( gstr, STR_NAME ); break;
-        }
-        if( order == GTK_SORT_DESCENDING )
-            g_string_prepend( gstr, STR_REVERSE );
-        pref_string_set( PREF_KEY_SORT_COLUMN, gstr->str );
-        g_string_free( gstr, TRUE );
-    }
+    return 0; /* FIXME */
 }
 
-void
-tr_core_set_sort_column_from_prefs( TrCore * core )
+static int
+compareByName( GtkTreeModel   * model,
+               GtkTreeIter    * a,
+               GtkTreeIter    * b,
+               gpointer         user_data UNUSED )
 {
-    char * val = pref_string_get( PREF_KEY_SORT_COLUMN );
-    char * freeme = val;
-    gint column;
-    GtkSortType order = GTK_SORT_ASCENDING;
-    if( g_str_has_prefix( val, STR_REVERSE ) ) {
-        order = GTK_SORT_DESCENDING;
-        val += strlen( STR_REVERSE );
+    int ret;
+    char *ca, *cb;
+    gtk_tree_model_get( model, a, MC_NAME_COLLATED, &ca, -1 );
+    gtk_tree_model_get( model, b, MC_NAME_COLLATED, &cb, -1 );
+    ret = strcmp( ca, cb );
+    g_free( cb );
+    g_free( ca );
+    return ret;
+}
+
+static int
+compareByProgress( GtkTreeModel   * model,
+                   GtkTreeIter    * a,
+                   GtkTreeIter    * b,
+                   gpointer         user_data UNUSED )
+{
+    int ret;
+    tr_torrent *ta, *tb;
+    const tr_stat *sa, *sb;
+    gtk_tree_model_get( model, a, MC_TORRENT_RAW, &ta, -1 );
+    gtk_tree_model_get( model, b, MC_TORRENT_RAW, &tb, -1 );
+    sa = tr_torrentStatCached( ta );
+    sb = tr_torrentStatCached( tb );
+    ret = compareDouble( sa->percentDone, sb->percentDone );
+    if( !ret )
+        ret = compareDouble( sa->ratio, sb->ratio );
+    return ret;
+}
+
+static int
+compareByState( GtkTreeModel   * model,
+                GtkTreeIter    * a,
+                GtkTreeIter    * b,
+                gpointer         user_data )
+{
+    int sa, sb, ret;
+
+    /* first by state */
+    gtk_tree_model_get( model, a, MC_STATUS, &sa, -1 );
+    gtk_tree_model_get( model, b, MC_STATUS, &sb, -1 );
+    ret = sa - sb;
+
+    /* second by progress */
+    if( !ret )
+        ret = compareByProgress( model, a, b, user_data );
+
+    return ret;
+}
+
+static int
+compareByTracker( GtkTreeModel   * model,
+                  GtkTreeIter    * a,
+                  GtkTreeIter    * b,
+                  gpointer         user_data UNUSED )
+{
+    const tr_torrent *ta, *tb;
+    gtk_tree_model_get( model, a, MC_TORRENT_RAW, &ta, -1 );
+    gtk_tree_model_get( model, b, MC_TORRENT_RAW, &tb, -1 );
+    return strcmp( tr_torrentInfo(ta)->primaryAddress,
+                   tr_torrentInfo(tb)->primaryAddress );
+}
+
+/***
+****
+***/
+
+
+static void
+setSort( TrCore * core, const char * mode, gboolean isReversed  )
+{
+    int col = MC_TORRENT_RAW;
+    GtkSortType type = isReversed ? GTK_SORT_ASCENDING : GTK_SORT_DESCENDING;
+    GtkTreeSortable * sortable = GTK_TREE_SORTABLE( core->model );
+
+    if( !strcmp( mode, "sort-by-activity" ) )
+        gtk_tree_sortable_set_sort_func( sortable, col, compareByActivity, NULL, NULL );
+    else if( !strcmp( mode, "sort-by-date-added" ) )
+        gtk_tree_sortable_set_sort_func( sortable, col, compareByDateAdded, NULL, NULL );
+    else if( !strcmp( mode, "sort-by-progress" ) )
+        gtk_tree_sortable_set_sort_func( sortable, col, compareByProgress, NULL, NULL );
+    else if( !strcmp( mode, "sort-by-state" ) )
+        gtk_tree_sortable_set_sort_func( sortable, col, compareByState, NULL, NULL );
+    else if( !strcmp( mode, "sort-by-tracker" ) )
+        gtk_tree_sortable_set_sort_func( sortable, col, compareByTracker, NULL, NULL );
+    else {
+        type = isReversed ? GTK_SORT_DESCENDING : GTK_SORT_ASCENDING;
+        gtk_tree_sortable_set_sort_func( sortable, col, compareByName, NULL, NULL );
+    }
+
+    gtk_tree_sortable_set_sort_column_id( sortable, col, type );
+}
+
+static void
+prefsChanged( TrCore * core, const char * key, gpointer data UNUSED )
+{
+    if( !strcmp( key, PREF_KEY_SORT_MODE ) || !strcmp( key, PREF_KEY_SORT_REVERSED ) )
+    {
+        char * mode = pref_string_get( PREF_KEY_SORT_MODE );
+        gboolean isReversed = pref_flag_get( PREF_KEY_SORT_REVERSED );
+        setSort( core, mode, isReversed );
+        g_free( mode );
+    }
+    else if( !strcmp( key, PREF_KEY_MAX_PEERS_GLOBAL ) )
+    {
+        const uint16_t val = pref_int_get( key );
+        tr_setGlobalPeerLimit( core->handle, val );
     }
-    if( !strcmp( val, STR_PROGRESS ) )
-        column = MC_PROG_D;
-    else /* default */
-        column = MC_NAME;
-    gtk_tree_sortable_set_sort_column_id ( GTK_TREE_SORTABLE( core->model ), column, order );
-    g_free( freeme );
 }
 
 static void
 tr_core_init( GTypeInstance * instance, gpointer g_class SHUTUP )
 {
+    tr_handle * h;
     TrCore * self = (TrCore *) instance;
     GtkListStore * store;
 
     /* column types for the model used to store torrent information */
     /* keep this in sync with the enum near the bottom of tr_core.h */
-    GType types[] =
-    {
-        /* info->name, info->totalSize, info->hashString, status, */
-        G_TYPE_STRING, G_TYPE_UINT64,   G_TYPE_STRING,    G_TYPE_INT,
-        /* error,   errorString,   percentComplete, percentDone,  rateDownload, rateUpload, */
-        G_TYPE_INT, G_TYPE_STRING, G_TYPE_FLOAT,    G_TYPE_FLOAT, G_TYPE_FLOAT, G_TYPE_FLOAT,
-        /* eta,     peersConnected, peersUploading, peersDownloading, seeders, */
-        G_TYPE_INT, G_TYPE_INT,     G_TYPE_INT,     G_TYPE_INT,       G_TYPE_INT,
-        /* leechers, completedFromTracker, downloaded,    uploaded */
-        G_TYPE_INT,  G_TYPE_INT,           G_TYPE_UINT64, G_TYPE_UINT64,
-        /* left,       TrTorrent object, ID for IPC */
-        G_TYPE_UINT64, TR_TORRENT_TYPE,  G_TYPE_INT,
+    GType types[] = {
+        G_TYPE_STRING,    /* name */
+        G_TYPE_STRING,    /* collated name */
+        G_TYPE_STRING,    /* hash string */
+        TR_TORRENT_TYPE,  /* TrTorrent object */
+        G_TYPE_POINTER,   /* tr_torrent* */
+        G_TYPE_INT,       /* tr_stat()->status */
+        G_TYPE_INT        /* ID for IPC */
     };
 
-#ifdef REFDBG
-    fprintf( stderr, "core    %p init\n", self );
-#endif
+    h = tr_initFull( "gtk",
+                     pref_flag_get( PREF_KEY_PEX ),
+                     pref_flag_get( PREF_KEY_NAT ),
+                     pref_int_get( PREF_KEY_PORT ),
+                     pref_flag_get( PREF_KEY_ENCRYPTED_ONLY )
+                         ? TR_ENCRYPTION_REQUIRED
+                         : TR_ENCRYPTION_PREFERRED,
+                     pref_flag_get( PREF_KEY_UL_LIMIT_ENABLED ),
+                     pref_int_get( PREF_KEY_UL_LIMIT ),
+                     pref_flag_get( PREF_KEY_DL_LIMIT_ENABLED ),
+                     pref_int_get( PREF_KEY_DL_LIMIT ),
+                     pref_int_get( PREF_KEY_MAX_PEERS_GLOBAL ),
+                     pref_int_get( PREF_KEY_MSGLEVEL ),
+                     TRUE );
 
     /* create the model used to store torrent data */
     g_assert( ALEN( types ) == MC_ROW_COUNT );
     store = gtk_list_store_newv( MC_ROW_COUNT, types );
-    g_signal_connect( store, "sort-column-changed", G_CALLBACK(onSortColumnChanged), NULL );
-
-    gtk_tree_sortable_set_sort_func( GTK_TREE_SORTABLE(store),
-                                     MC_PROG_D,
-                                     compareProgress,
-                                     NULL, NULL );
 
     self->model    = GTK_TREE_MODEL( store );
-    self->handle   = tr_init( "gtk" );
+    self->handle   = h;
     self->nextid   = 1;
     self->quitting = FALSE;
     self->disposed = FALSE;
@@ -326,7 +378,7 @@ tr_core_get_type( void )
 {
     static GType type = 0;
 
-    if( 0 == type )
+    if( !type )
     {
         static const GTypeInfo info =
         {
@@ -354,7 +406,15 @@ tr_core_get_type( void )
 TrCore *
 tr_core_new( void )
 {
-    return g_object_new( TR_CORE_TYPE, NULL );
+    TrCore * core = TR_CORE( g_object_new( TR_CORE_TYPE, NULL ) );
+
+    /* init from prefs & listen to pref changes */
+    prefsChanged( core, PREF_KEY_SORT_MODE, NULL );
+    prefsChanged( core, PREF_KEY_SORT_REVERSED, NULL );
+    prefsChanged( core, PREF_KEY_MAX_PEERS_GLOBAL, NULL );
+    g_signal_connect( core, "prefs-changed", G_CALLBACK(prefsChanged), NULL );
+
+    return core;
 }
 
 GtkTreeModel *
@@ -373,86 +433,78 @@ tr_core_handle( TrCore * self )
     return self->disposed ? NULL : self->handle;
 }
 
-void
-tr_core_shutdown( TrCore * self )
+static char*
+doCollate( const char * in )
 {
-    GtkTreeIter iter;
+    const char * end = in + strlen( in );
+    char * casefold;
+    char * ret;
 
-    TR_IS_CORE( self );
+    while( in < end ) {
+        const gunichar ch = g_utf8_get_char( in );
+        if (!g_unichar_isalnum (ch)) // eat everything before the first alnum
+            in += g_unichar_to_utf8( ch, NULL );
+        else
+            break;
+    }
 
-    if( self->disposed )
-        return;
+    if ( in == end )
+        return g_strdup ("");
 
-    g_assert( !self->quitting );
-    self->quitting = TRUE;
-
-    /* try to stop all the torrents nicely */
-    if ( gtk_tree_model_get_iter_first( self->model, &iter) ) do {
-        TrTorrent * tor;
-        gtk_tree_model_get( self->model, &iter, MC_TORRENT, &tor, -1 );
-        tr_torrent_sever( tor );
-        g_object_unref( tor );
-    } while( gtk_list_store_remove( GTK_LIST_STORE(self->model), &iter ) );
-
-    /* shut down nat traversal */
-    tr_natTraversalEnable( self->handle, 0 );
-}
-
-gboolean
-tr_core_quiescent( TrCore * self )
-{
-    const tr_handle_status * hstat;
-
-    TR_IS_CORE( self );
-    g_assert( self->quitting );
-
-    if( self->disposed )
-        return TRUE;
-
-    if ( tr_torrentCount( self->handle ) != 0 )
-        return FALSE;
-
-    hstat = tr_handleStatus( self->handle );
-    return TR_NAT_TRAVERSAL_DISABLED == hstat->natTraversalStatus;
+    casefold = g_utf8_casefold( in, end-in );
+    ret = g_utf8_collate_key( casefold, -1 );
+    g_free( casefold );
+    return ret;
 }
 
 static void
 tr_core_insert( TrCore * self, TrTorrent * tor )
 {
-    GtkTreeIter iter;
-    const tr_info * inf;
-
-    gtk_list_store_append( GTK_LIST_STORE( self->model ), &iter );
-    inf = tr_torrent_info( tor );
-    gtk_list_store_set( GTK_LIST_STORE( self->model ), &iter,
-                        MC_NAME,    inf->name,
-                        MC_SIZE,    inf->totalSize,
-                        MC_HASH,    inf->hashString,
-                        MC_TORRENT, tor,
-                        MC_ID,      self->nextid,
-                        -1);
-    g_object_unref( tor );
+    const tr_info * inf = tr_torrent_info( tor );
+    const tr_stat * torStat = tr_torrent_stat( tor );
+    char * collated = doCollate( inf->name );
+    GtkTreeIter unused;
+    gtk_list_store_insert_with_values( GTK_LIST_STORE( self->model ), &unused, 0, 
+                                       MC_NAME,          inf->name,
+                                       MC_NAME_COLLATED, collated,
+                                       MC_HASH,          inf->hashString,
+                                       MC_TORRENT,       tor,
+                                       MC_TORRENT_RAW,   tor->handle,
+                                       MC_STATUS,        torStat->status,
+                                       MC_ID,            self->nextid,
+                                       -1);
     self->nextid++;
+    g_object_unref( tor );
+    g_free( collated );
 }
 
 int
-tr_core_load( TrCore * self, gboolean paused )
+tr_core_load( TrCore * self, gboolean forcePaused )
 {
     int i;
     int count = 0;
     tr_torrent ** torrents;
     char * path;
+    tr_ctor * ctor;
 
     TR_IS_CORE( self );
 
     path = getdownloaddir( );
 
-    torrents = tr_loadTorrents ( self->handle, path, paused, &count );
+    ctor = tr_ctorNew( self->handle );
+    if( forcePaused )
+        tr_ctorSetPaused( ctor, TR_FORCE, TRUE );
+    tr_ctorSetDestination( ctor, TR_FALLBACK, path );
+    tr_ctorSetMaxConnectedPeers( ctor, TR_FALLBACK, pref_int_get( PREF_KEY_MAX_PEERS_PER_TORRENT ) );
+
+    torrents = tr_loadTorrents ( self->handle, ctor, &count );
     for( i=0; i<count; ++i )
         tr_core_insert( self, tr_torrent_new_preexisting( torrents[i] ) );
-    tr_free( torrents );
 
+    tr_free( torrents );
+    tr_ctorFree( ctor );
     g_free( path );
+
     return count;
 }
 
@@ -595,47 +647,52 @@ tr_core_delete_torrent( TrCore * self, GtkTreeIter * iter )
     gtk_list_store_remove( GTK_LIST_STORE( self->model ), iter );
     tr_torrentRemoveSaved( tr_torrent_handle( tor ) );
 
-    tr_torrent_sever( tor );
+    g_object_unref( G_OBJECT( tor ) );
+}
+
+static gboolean
+update_foreach( GtkTreeModel * model,
+                GtkTreePath  * path UNUSED,
+                GtkTreeIter  * iter,
+                gpointer       data UNUSED)
+{
+    TrTorrent * gtor;
+    int oldStatus;
+    const tr_stat * torStat;
+    gtk_tree_model_get( model, iter, MC_TORRENT, &gtor,
+                                     MC_STATUS, &oldStatus,
+                                     -1 );
+
+    torStat = tr_torrent_stat( gtor );
+
+    if( oldStatus != (int) torStat->status )
+        gtk_list_store_set( GTK_LIST_STORE( model ), iter,
+                            MC_STATUS, torStat->status,
+                            -1 );
+
+    tr_torrent_check_seeding_cap ( gtor );
+
+    g_object_unref( gtor );
+    return FALSE;
 }
 
 void
 tr_core_update( TrCore * self )
 {
-    GtkTreeIter iter;
-    TrTorrent * tor;
-    const tr_stat * st;
+    int column;
+    GtkSortType order;
+    GtkTreeSortable * sortable;
 
-    TR_IS_CORE( self );
+    /* pause sorting */
+    sortable = GTK_TREE_SORTABLE( self->model );
+    gtk_tree_sortable_get_sort_column_id( sortable, &column, &order );
+    gtk_tree_sortable_set_sort_column_id( sortable, GTK_TREE_SORTABLE_UNSORTED_SORT_COLUMN_ID, order );
 
-    if( gtk_tree_model_get_iter_first( self->model, &iter ) ) do
-    {
-        gtk_tree_model_get( self->model, &iter, MC_TORRENT, &tor, -1 );
-        st = tr_torrent_stat( tor );
-        g_object_unref( tor );
-        tr_torrent_check_seeding_cap ( tor );
+    /* refresh the model */
+    gtk_tree_model_foreach( self->model, update_foreach, NULL );
 
-        /* XXX find out if setting the same data emits changed signal */
-        gtk_list_store_set( GTK_LIST_STORE( self->model ), &iter,
-                            MC_STAT,        st->status,
-                            MC_ERR,         st->error,
-                            MC_TERR,        st->errorString,
-                            MC_PROG_C,      st->percentComplete,
-                            MC_PROG_D,      st->percentDone,
-                            MC_DRATE,       st->rateDownload,
-                            MC_URATE,       st->rateUpload,
-                            MC_ETA,         st->eta,
-                            MC_PEERS,       st->peersConnected,
-                            MC_UPEERS,      st->peersGettingFromUs,
-                            MC_DPEERS,      st->peersSendingToUs,
-                            MC_SEED,        st->seeders,
-                            MC_LEECH,       st->leechers,
-                            MC_DONE,        st->completedFromTracker,
-                            MC_DOWN,        st->downloadedEver,
-                            MC_UP,          st->uploadedEver,
-                            MC_LEFT,        st->leftUntilDone,
-                            -1 );
-    }
-    while( gtk_tree_model_iter_next( self->model, &iter ) );
+    /* resume sorting */
+    gtk_tree_sortable_set_sort_column_id( sortable, column, order );
 }
 
 void
