@@ -1,7 +1,7 @@
 /******************************************************************************
- * $Id: tr_icon.c 3223 2007-09-28 15:22:29Z charles $
+ * $Id: tr_icon.c 4976 2008-02-07 21:43:39Z charles $
  *
- * Copyright (c) 2006-2007 Transmission authors and contributors
+ * Copyright (c) 2006-2008 Transmission authors and contributors
  *
  * Permission is hereby granted, free of charge, to any person obtaining a
  * copy of this software and associated documentation files (the "Software"),
@@ -59,7 +59,7 @@ popup ( GtkStatusIcon  * self,
 gpointer
 tr_icon_new( void )
 {
-    GtkStatusIcon * ret = gtk_status_icon_new_from_stock ("transmission-logo");
+    GtkStatusIcon * ret = gtk_status_icon_new_from_icon_name( "transmission" );
     g_signal_connect( ret, "activate", G_CALLBACK( activated ), NULL );
     g_signal_connect( ret, "popup-menu", G_CALLBACK( popup ), NULL );
     return ret;
