@@ -1,7 +1,7 @@
 /******************************************************************************
- * $Id: tr_icon.h 2149 2007-06-18 03:40:41Z livings124 $
+ * $Id: tr_icon.h 4811 2008-01-23 16:22:54Z charles $
  *
- * Copyright (c) 2006-2007 Transmission authors and contributors
+ * Copyright (c) 2006-2008 Transmission authors and contributors
  *
  * Permission is hereby granted, free of charge, to any person obtaining a
  * copy of this software and associated documentation files (the "Software"),
@@ -29,9 +29,6 @@
 
 #if GTK_CHECK_VERSION(2,10,0)
 #define STATUS_ICON_SUPPORTED
-#define status_icon_supported() (TRUE)
-#else
-#define status_icon_supported() (FALSE)
 #endif
 
 gpointer tr_icon_new( void );
