@@ -1,7 +1,7 @@
 /******************************************************************************
- * $Id: tr_torrent.h 3381 2007-10-12 20:50:03Z charles $
+ * $Id: tr_torrent.h 4404 2008-01-01 17:20:20Z charles $
  *
- * Copyright (c) 2006-2007 Transmission authors and contributors
+ * Copyright (c) 2006-2008 Transmission authors and contributors
  *
  * Permission is hereby granted, free of charge, to any person obtaining a
  * copy of this software and associated documentation files (the "Software"),
@@ -50,8 +50,6 @@ struct _TrTorrent {
   GObject parent;
   tr_torrent *handle;
   char *delfile;
-  tr_stat stat;
-  time_t lastStatTime;
 
   /* FIXME: hm, are these heavyweight enough to deserve their own properties? */
   gboolean severed;
