@@ -1,7 +1,7 @@
 /******************************************************************************
- * $Id: util.c 3406 2007-10-13 23:22:46Z charles $
+ * $Id: util.c 4749 2008-01-20 03:25:12Z charles $
  *
- * Copyright (c) 2005-2007 Transmission authors and contributors
+ * Copyright (c) 2005-2008 Transmission authors and contributors
  *
  * Permission is hereby granted, free of charge, to any person obtaining a
  * copy of this software and associated documentation files (the "Software"),
@@ -38,8 +38,6 @@
 #include "conf.h"
 #include "util.h"
 
-#define BESTDECIMAL(d) ( (d)<100 ? 1 : 0 )
-
 static void
 errcb(GtkWidget *wind, int resp, gpointer data);
 
@@ -52,89 +50,85 @@ tr_strcmp( const char * a, const char * b )
     return 0;
 }
 
-static const char *sizestrs[] = {
-  N_("B"), N_("KiB"), N_("MiB"), N_("GiB"), N_("TiB"), N_("PiB"), N_("EiB"),
-};
-
-char *
-readablesize(guint64 size) {
-  int ii;
-  double small = size;
-
-  if( !size )
-    return g_strdup_printf( _("None") );
-
-  for(ii = 0; ii + 1 < ALEN(sizestrs) && 1024.0 <= small / 1024.0; ii++)
-    small /= 1024.0;
-
-  if(1024.0 <= small) {
-    small /= 1024.0;
-    ii++;
-  }
-
-  return g_strdup_printf("%.*f %s", BESTDECIMAL(small), small,
-                         gettext(sizestrs[ii]));
-}
-
-char *
-readablespeed (double KiBps)
+char*
+tr_strlsize( char * buf, guint64 size, size_t buflen )
 {
-  const guint64 bps = KiBps * 1024;
-  char * str = readablesize (bps);
-  char * ret = bps ? g_strdup_printf ("%s/s", str) : g_strdup( str );
-  g_free (str);
-  return ret;
+    if( !size )
+        g_strlcpy( buf, _("None"), buflen );
+    else {
+        static const char *units[] = {
+            N_("B"), N_("KiB"), N_("MiB"), N_("GiB"), N_("TiB"),
+            N_("PiB"), N_("EiB"), N_("ZiB"), N_("YiB")
+        };
+        unsigned int i;
+        double small = size;
+        for( i=0; i<G_N_ELEMENTS(units) && (small>=1024.0); ++i )
+            small /= 1024.0;
+        if( i < 2 ) /* B & KiB */
+            g_snprintf( buf, buflen, "%d %s", (int)small, _(units[i]) );
+        else
+            g_snprintf( buf, buflen, "%.1f %s", small, _(units[i]) );
+    }
+    return buf;
+}
+
+char*
+tr_strlspeed( char * buf, double KiBps, size_t buflen )
+{
+    const guint64 bps = KiBps * 1024;
+    if( !bps )
+        g_strlcpy( buf, _("None"), buflen );
+    else {
+        char bbuf[64];
+        tr_strlsize( bbuf, (guint64)(KiBps*1024), sizeof(bbuf) );
+        g_snprintf( buf, buflen, _("%s/s"), bbuf );
+    }
+    return buf;
 }
- 
 
 #define SECONDS(s)              ((s) % 60)
 #define MINUTES(s)              ((s) / 60 % 60)
 #define HOURS(s)                ((s) / 60 / 60 % 24)
 #define DAYS(s)                 ((s) / 60 / 60 / 24 % 7)
 
-char *
-readabletime(int secs) {
-  if(60 > secs)
-    return g_strdup_printf(_("%i %s"),
-      SECONDS(secs), ngettext("sec", "secs", SECONDS(secs)));
-  else if(60 * 60 > secs)
-    return g_strdup_printf(_("%i %s %i %s"),
-      MINUTES(secs), ngettext("min", "mins", MINUTES(secs)),
-      SECONDS(secs), ngettext("sec", "secs", SECONDS(secs)));
-  else if(60 * 60 * 24 > secs)
-    return g_strdup_printf(_("%i %s %i %s"),
-      HOURS(secs),   ngettext("hr", "hrs", HOURS(secs)),
-      MINUTES(secs), ngettext("min", "mins", MINUTES(secs)));
-  else
-    return g_strdup_printf(_("%i %s %i %s"),
-      DAYS(secs),    ngettext("day", "days", DAYS(secs)),
-      HOURS(secs),   ngettext("hr", "hrs", HOURS(secs)));
+char*
+tr_strltime( char * buf, int secs, size_t buflen )
+{
+    if( secs < 60 )
+    {
+        g_snprintf( buf, buflen, _( "%i %s" ),
+                    SECONDS(secs), ngettext("sec", "secs", SECONDS(secs)));
+    }
+    else if( secs < 60*60 )
+    {
+        g_snprintf( buf, buflen, _("%i %s %i %s"),
+                    MINUTES(secs), ngettext("min", "mins", MINUTES(secs)),
+                    SECONDS(secs), ngettext("sec", "secs", SECONDS(secs)));
+    }
+    else if( secs < 60*60*24 )
+    {
+        g_snprintf( buf, buflen, _("%i %s %i %s"),
+                    HOURS(secs),   ngettext("hr", "hrs", HOURS(secs)),
+                    MINUTES(secs), ngettext("min", "mins", MINUTES(secs)));
+    }
+    else
+    {
+        g_snprintf( buf, buflen, _("%i %s %i %s"),
+                    DAYS(secs),  ngettext("day", "days", DAYS(secs)),
+                    HOURS(secs), ngettext("hr", "hrs", HOURS(secs)));
+    }
+
+    return buf;
 }
 
 char *
 rfc822date (guint64 epoch_msec)
 {
-  const time_t secs = epoch_msec / 1000;
-  const struct tm tm = *localtime (&secs);
-  char buf[128];
-  strftime (buf, sizeof(buf), "%a, %d %b %Y %T %Z", &tm);
-  return g_strdup (buf);
-}
-
-char *
-ratiostr(guint64 down, guint64 up) {
-  double ratio;
-
-  if(0 == up && 0 == down)
-    return g_strdup(_("N/A"));
-
-  if(0 == down)
-    /* this is a UTF-8 infinity symbol */
-    return g_strdup("\xE2\x88\x9E");
-
-  ratio = (double)up / (double)down;
-
-  return g_strdup_printf("%.*f", BESTDECIMAL(ratio), ratio);
+    const time_t secs = epoch_msec / 1000;
+    const struct tm tm = *localtime (&secs);
+    char buf[128];
+    strftime( buf, sizeof(buf), "%a, %d %b %Y %T %Z", &tm );
+    return g_locale_to_utf8( buf, -1, NULL, NULL, NULL );
 }
 
 gboolean
@@ -443,3 +437,14 @@ on_tree_view_button_pressed (GtkWidget       * view,
   return FALSE;
 }
 
+gpointer
+tr_object_ref_sink( gpointer object )
+{
+#if GLIB_CHECK_VERSION(2,10,0)
+    g_object_ref_sink( object );
+#else
+    g_object_ref( object );
+    gtk_object_sink( GTK_OBJECT( object ) );
+#endif
+    return object;
+}
