@@ -1,7 +1,7 @@
 /******************************************************************************
- * $Id: util.h 3380 2007-10-12 19:53:30Z charles $
+ * $Id: util.h 4676 2008-01-15 17:55:10Z charles $
  *
- * Copyright (c) 2005-2007 Transmission authors and contributors
+ * Copyright (c) 2005-2008 Transmission authors and contributors
  *
  * Permission is hereby granted, free of charge, to any person obtaining a
  * copy of this software and associated documentation files (the "Software"),
@@ -44,30 +44,18 @@ enum tr_torrent_action { TR_TOR_LEAVE, TR_TOR_COPY, TR_TOR_MOVE };
 /* used for a callback function with a data parameter */
 typedef void (*callbackfunc_t)(void*);
 
-/* return a human-readable string for the size given in bytes.
-   the string must be g_free()d */
-char *
-readablesize(guint64 size);
+/* return a human-readable string for the size given in bytes. */
+char* tr_strlsize( char * buf, guint64 size, size_t buflen );
 
-/* return a human-readable string for the transfer rate given in bytes.
-   the string must be g_free()d */
-char *
-readablespeed (double KiBps);
+/* return a human-readable string for the transfer rate given in bytes. */
+char* tr_strlspeed (char * buf, double KiBps, size_t buflen );
 
-
-/* return a human-readable string for the time given in seconds.
-   the string must be g_free()d */
-char *
-readabletime(int secs);
+/* return a human-readable string for the time given in seconds. */
+char* tr_strltime( char * buf, int secs, size_t buflen );
 
 char *
 rfc822date (guint64 epoch_msec);
 
-/* returns a string representing the download ratio.
-   the string must be g_free()d */
-char *
-ratiostr(guint64 down, guint64 up);
-
 /* create a directory and any missing parent directories */
 gboolean
 mkdir_p(const char *name, mode_t mode);
@@ -142,6 +130,8 @@ on_tree_view_button_pressed (GtkWidget       * view,
                              GdkEventButton  * event,
                              gpointer          unused);
 
+gpointer tr_object_ref_sink (gpointer object);
+
 #endif /* GTK_MAJOR_VERSION */
 
 #endif /* TG_UTIL_H */
