@@ -1,7 +1,7 @@
 /******************************************************************************
- * $Id: completion.h 3178 2007-09-26 01:55:04Z charles $
+ * $Id: completion.h 4404 2008-01-01 17:20:20Z charles $
  *
- * Copyright (c) 2005 Transmission authors and contributors
+ * Copyright (c) 2005-2008 Transmission authors and contributors
  *
  * Permission is hereby granted, free of charge, to any person obtaining a
  * copy of this software and associated documentation files (the "Software"),
@@ -27,11 +27,11 @@
 
 #include "transmission.h"
 
+struct tr_bitfield;
 typedef struct tr_completion tr_completion;
 
 tr_completion  * tr_cpInit( tr_torrent * );
 void             tr_cpClose( tr_completion * );
-void             tr_cpReset( tr_completion * );
 
 /* General */
 
