@@ -1,7 +1,7 @@
 /******************************************************************************
- * $Id: inout.h 3284 2007-10-04 19:58:30Z charles $
+ * $Id: inout.h 4776 2008-01-21 02:07:55Z charles $
  *
- * Copyright (c) 2005-2006 Transmission authors and contributors
+ * Copyright (c) 2005-2008 Transmission authors and contributors
  *
  * Permission is hereby granted, free of charge, to any person obtaining a
  * copy of this software and associated documentation files (the "Software"),
@@ -27,25 +27,23 @@
 
 struct tr_torrent;
 
-typedef struct tr_io tr_io;
-
-/***********************************************************************
- * tr_ioRead, tr_ioWrite
- ***********************************************************************
- * Reads or writes the block specified by the piece index, the offset in
- * that piece and the size of the block. Returns 0 if successful, 
- * TR_ERROR_ASSERT if the parameters are incorrect, one of the
- * TR_ERROR_IO_* otherwise.
- **********************************************************************/
+/**
+ * Reads the block specified by the piece index, offset, and length.
+ * @return 0 on success, TR_ERROR_ASSERT if the arguments are incorrect,
+ * or TR_ERROR_IO_* otherwise.
+ */
 int tr_ioRead  ( struct tr_torrent*, int index, int begin, int len, uint8_t * );
-int tr_ioWrite ( struct tr_torrent *, int index, int begin, int len, uint8_t * );
+
+/**
+ * Writes the block specified by the piece index, offset, and length.
+ * @return 0 on success, TR_ERROR_ASSERT if the arguments are incorrect,
+ * or TR_ERROR_IO_* otherwise.
+ */
+int tr_ioWrite ( struct tr_torrent *, int index, int begin, int len, const uint8_t * );
 
 /* hashes the specified piece and updates the completion accordingly. */
 int tr_ioHash ( tr_torrent*, int piece );
 
-/* close all the files associated with this torrent*/
-void tr_ioClose( const tr_torrent * );
-
 /**
 ***
 **/
