@@ -1,7 +1,7 @@
 /******************************************************************************
- * $Id: ipcparse.c 3486 2007-10-20 22:07:21Z charles $
+ * $Id: ipcparse.c 4538 2008-01-07 04:28:41Z livings124 $
  *
- * Copyright (c) 2007 Joshua Elsasser
+ * Copyright (c) 2007-2008 Joshua Elsasser
  *
  * Permission is hereby granted, free of charge, to any person obtaining a
  * copy of this software and associated documentation files (the "Software"),
@@ -30,6 +30,7 @@
 #include <string.h>
 
 #include "transmission.h"
+#include "utils.h"
 
 #include "ipcparse.h"
 #include "bsdtree.h"
@@ -140,11 +141,13 @@ static struct msg gl_msgs[] =
     { "automap",             2, IPC_MSG_AUTOMAP,      RB_ENTRY_INITIALIZER() },
     { "autostart",           2, IPC_MSG_AUTOSTART,    RB_ENTRY_INITIALIZER() },
     { "bad-format",          2, IPC_MSG_BAD,          RB_ENTRY_INITIALIZER() },
+    { "encryption",          2, IPC_MSG_CRYPTO,       RB_ENTRY_INITIALIZER() },
     { "directory",           2, IPC_MSG_DIR,          RB_ENTRY_INITIALIZER() },
     { "downlimit",           2, IPC_MSG_DOWNLIMIT,    RB_ENTRY_INITIALIZER() },
     { "failed",              2, IPC_MSG_FAIL,         RB_ENTRY_INITIALIZER() },
     { "get-automap",         2, IPC_MSG_GETAUTOMAP,   RB_ENTRY_INITIALIZER() },
     { "get-autostart",       2, IPC_MSG_GETAUTOSTART, RB_ENTRY_INITIALIZER() },
+    { "get-encryption",      2, IPC_MSG_GETCRYPTO,    RB_ENTRY_INITIALIZER() },
     { "get-directory",       2, IPC_MSG_GETDIR,       RB_ENTRY_INITIALIZER() },
     { "get-downlimit",       2, IPC_MSG_GETDOWNLIMIT, RB_ENTRY_INITIALIZER() },
     { "get-info",            2, IPC_MSG_GETINFO,      RB_ENTRY_INITIALIZER() },
@@ -361,40 +364,24 @@ ipc_initval( struct ipc_info * info, enum ipc_msg id, int64_t tag,
 }
 
 uint8_t *
-ipc_mkval( benc_val_t * pk, size_t * len )
+ipc_mkval( benc_val_t * pk, size_t * setmeSize )
 {
-    char * buf, hex[IPC_MIN_MSG_LEN+1];
-    int    used, max;
+    int bencSize = 0;
+    char * benc = tr_bencSave( pk, &bencSize );
+    uint8_t * ret = NULL;
 
-    used = IPC_MIN_MSG_LEN;
-    max  = IPC_MIN_MSG_LEN;
-    buf  = malloc( IPC_MIN_MSG_LEN );
-    if( NULL == buf )
-    {
-        return NULL;
-    }
-
-    if( tr_bencSave( pk, &buf, &used, &max ) )
-    {
-        SAFEFREE( buf );
-        return NULL;
-    }
-
-    /* ok, this check is pretty laughable */
-    if( IPC_MAX_MSG_LEN < used )
-    {
-        free( buf );
+    if( bencSize > IPC_MAX_MSG_LEN )
         errno = EFBIG;
-        return NULL;
+    else {
+        const size_t size = IPC_MIN_MSG_LEN + bencSize;
+        ret = tr_new( uint8_t, size );
+        snprintf( (char*)ret, size, "%0*X", IPC_MIN_MSG_LEN, bencSize );
+        memcpy( ret + IPC_MIN_MSG_LEN, benc, bencSize );
+        *setmeSize = size;
     }
 
-    assert( 0 <= used );
-    snprintf( hex, sizeof hex, "%0*X",
-              IPC_MIN_MSG_LEN, used - IPC_MIN_MSG_LEN );
-    memcpy( buf, hex, IPC_MIN_MSG_LEN );
-    *len = used;
-
-    return ( uint8_t * )buf;
+    tr_free( benc );
+    return ret;
 }
 
 uint8_t *
@@ -774,16 +761,12 @@ ipc_addstat( benc_val_t * list, int tor,
                 error = st->error;
                 if( TR_OK == error )
                 {
-                    tr_bencInitStr( item, NULL, 0, 1 );
+                    tr_bencInitStr( item, "", -1, 1 );
                 }
                 else if( TR_ERROR_ISSET( TR_ERROR_ASSERT, error ) )
                 {
                     tr_bencInitStr( item, "assert", -1, 1 );
                 }
-                else if( TR_ERROR_ISSET( TR_ERROR_IO_PARENT, error ) )
-                {
-                    tr_bencInitStr( item, "io-parent", -1, 1 );
-                }
                 else if( TR_ERROR_ISSET( TR_ERROR_IO_PERMISSIONS, error ) )
                 {
                     tr_bencInitStr( item, "io-permissions", -1, 1 );
@@ -818,9 +801,13 @@ ipc_addstat( benc_val_t * list, int tor,
                 }
                 break;
             case IPC_ST_ERRMSG:
-                if( '\0' == st->errorString[0] )
+                if( TR_OK == st->error )
                 {
-                    tr_bencInitStr( item, NULL, 0, 1 );
+                    tr_bencInitStr( item, "", -1, 1 );
+                }
+                else if( '\0' == st->errorString[0] )
+                {
+                    tr_bencInitStr( item, "other", -1, 1 );
                 }
                 else if( tr_bencInitStrDup( item, st->errorString ) )
                 {
@@ -852,7 +839,7 @@ ipc_addstat( benc_val_t * list, int tor,
                                 st->peersFrom[TR_PEER_FROM_PEX] );
                 break;
             case IPC_ST_PEERTOTAL:
-                tr_bencInitInt( item, st->peersKnown );
+                tr_bencInitInt( item, st->peersConnected );
                 break;
             case IPC_ST_PEERUP:
                 tr_bencInitInt( item, st->peersGettingFromUs );
@@ -873,7 +860,7 @@ ipc_addstat( benc_val_t * list, int tor,
                 {
                     tr_bencInitStr( item, "downloading", -1, 1 );
                 }
-                else if( TR_STATUS_SEED & st->status )
+                else if( ( TR_STATUS_DONE | TR_STATUS_SEED ) & st->status )
                 {
                     tr_bencInitStr( item, "seeding", -1, 1 );
                 }
