@@ -1,7 +1,7 @@
 /******************************************************************************
- * $Id: transmission.h 3573 2007-10-26 03:02:35Z charles $
+ * $Id: transmission.h 4854 2008-01-28 17:53:32Z charles $
  *
- * Copyright (c) 2005-2007 Transmission authors and contributors
+ * Copyright (c) 2005-2008 Transmission authors and contributors
  *
  * Permission is hereby granted, free of charge, to any person obtaining a
  * copy of this software and associated documentation files (the "Software"),
@@ -60,7 +60,7 @@ extern "C" {
 #define TR_PATH_DELIMITER_STR "/"
 #endif
 
-#define TR_DEFAULT_PORT   9090
+#define TR_DEFAULT_PORT   51413
 
 enum
 {
@@ -80,7 +80,6 @@ enum
 #define TR_ERROR_ASSERT             0x82000000
 /* I/O errors */
 #define TR_ERROR_IO_MASK            0x000000FF
-#define TR_ERROR_IO_PARENT          0x80000001
 #define TR_ERROR_IO_PERMISSIONS     0x80000002
 #define TR_ERROR_IO_SPACE           0x80000004
 #define TR_ERROR_IO_FILE_TOO_BIG    0x80000008
@@ -97,8 +96,8 @@ enum
 /***********************************************************************
  * tr_init
  ***********************************************************************
- * Initializes a libtransmission instance. Returns a obscure handle to
- * be passed to all functions below. The tag argument is a short string
+ * Initializes a libtransmission instance and returns an opaque handle
+ * to be passed to functions below. The tag argument is a short string
  * unique to the program invoking tr_init(), it is currently used as
  * part of saved torrent files' names to prevent one frontend from
  * deleting a torrent used by another. The following tags are used:
@@ -109,7 +108,46 @@ typedef struct tr_handle tr_handle;
 
 tr_handle * tr_init( const char * tag );
 
-typedef struct tr_tracker_info tr_tracker_info;
+/* shut down a libtransmission instance created by tr_init(). */
+void tr_close( tr_handle * );
+
+tr_handle * tr_initFull( const char * tag,
+                         int          isPexEnabled,
+                         int          isNatEnabled,
+                         int          publicPort,
+                         int          encryptionMode,
+                         int          isUploadLimitEnabled,
+                         int          uploadLimit,
+                         int          isDownloadLimitEnabled,
+                         int          downloadLimit,
+                         int          globalPeerLimit,
+                         int          messageLevel,
+                         int          isMessageQueueingEnabled );
+
+
+/**
+***
+**/
+
+typedef struct tr_session_stats
+{
+    uint64_t uploadedBytes;   /* total up */
+    uint64_t downloadedBytes; /* total down */
+    double ratio;             /* TR_RATIO_INF, TR_RATIO_NA, or total up/down */
+    uint64_t filesAdded;      /* number of files added */
+    uint64_t sessionCount;    /* program started N times */
+    uint64_t secondsActive;   /* how long Transmisson's been running */
+}
+tr_session_stats;
+
+/* stats from the current and past sessions. */
+void tr_getCumulativeSessionStats( const tr_handle   * handle,
+                                   tr_session_stats  * setme );
+
+/* stats from the current session. */
+void tr_getSessionStats( const tr_handle   * handle,
+                         tr_session_stats  * setme );
+
 
 /**
 ***
@@ -117,6 +155,7 @@ typedef struct tr_tracker_info tr_tracker_info;
 
 typedef enum
 {
+    TR_PLAINTEXT_PREFERRED,
     TR_ENCRYPTION_PREFERRED,
     TR_ENCRYPTION_REQUIRED
 }
@@ -193,18 +232,10 @@ tr_handle_status * tr_handleStatus( tr_handle * );
  ***********************************************************************
  * Returns the count of open torrents
  **********************************************************************/
-int tr_torrentCount( tr_handle * h );
+int tr_torrentCount( const tr_handle * h );
 
 
-/***********************************************************************
- * tr_torrentIterate
- ***********************************************************************
- * Iterates on open torrents
- **********************************************************************/
 typedef struct tr_torrent tr_torrent;
-typedef void (*tr_callback_t) ( tr_torrent *, void * );
-void tr_torrentIterate( tr_handle *, tr_callback_t, void * );
-
 
 /***********************************************************************
 *** Speed Limits
@@ -247,6 +278,24 @@ void tr_getGlobalSpeedLimit( tr_handle * handle,
                              int         * setme_is_enabled,
                              int         * setme_KiBsec );
 
+
+/***********************************************************************
+***  Peer Limits
+**/
+
+void tr_torrentSetMaxConnectedPeers( tr_torrent  * tor,
+                                     uint16_t      maxConnectedPeers);
+
+uint16_t tr_torrentGetMaxConnectedPeers( const tr_torrent  * tor );
+
+void tr_setGlobalPeerLimit( tr_handle * handle,
+                            uint16_t    maxGlobalPeers );
+
+uint16_t tr_getGlobalPeerLimit( const tr_handle * handle );
+
+
+
+
 /***********************************************************************
  * Torrent Priorities
  **********************************************************************/
@@ -292,14 +341,123 @@ void tr_torrentSetFileDLs ( tr_torrent   * tor,
  **********************************************************************/
 void tr_torrentRates( tr_handle *, float *, float * );
 
-/***********************************************************************
- * tr_close
- ***********************************************************************
- * Frees memory allocated by tr_init.
- **********************************************************************/
-void tr_close( tr_handle * );
 
 
+/**
+ *  Torrent Instantiation
+ *
+ *  Instantiating a tr_torrent has gotten a lot more complaticated as we
+ *  add more options to them.  At the worst point there were four functions
+ *  to check metainfo, and four (or five) functions to create tr_torrents.
+ *  They all did mostly the same thing, but each was just *slightly* different.
+ *
+ *  To remedy this, a Torrent Constructor (struct tr_ctor) has been introduced:
+ *  + Simplifies the API down to two (non-deprecated) functions.
+ *  + You can set the fields you want; the system sets defaults for the rest.
+ *  + You can specify whether or not your fields should supercede fastresume's.
+ *  + We can add new features to tr_ctor without breaking tr_torrentNew()'s API.
+ *
+ *  All the tr_ctor{Get,Set}*() functions with a return value return 
+ *  an error number, or zero if no error occurred.
+ *
+ *  You must call one of the SetMetainfo() functions before creating
+ *  a torrent with a tr_ctor.  The other functions are optional.
+ *
+ *  You can reuse a single tr_ctor to create a batch of torrents --
+ *  just call one of the SetMetainfo() functions between each
+ *  tr_torrentNew() call.
+ *
+ *  Every call to tr_ctorSetMetainfo*() frees the previous metainfo.
+ */
+
+typedef enum
+{
+    TR_FALLBACK, /* indicates the ctor value should be used only
+                    in case of missing fastresume settings */
+
+    TR_FORCE, /* indicates the ctor value should be used
+                 regardless of what's in the fastresume settings */
+}
+tr_ctorMode;
+
+typedef struct tr_ctor tr_ctor;
+struct benc_val_s;
+
+tr_ctor* tr_ctorNew                    ( const tr_handle  * handle);
+
+void     tr_ctorFree                   ( tr_ctor        * ctor );
+
+void     tr_ctorSetSave                ( tr_ctor        * ctor,
+                                         int              saveMetadataInOurTorrentsDir );
+
+int      tr_ctorSetMetainfo            ( tr_ctor        * ctor,
+                                         const uint8_t  * metainfo,
+                                         size_t           len );
+
+int      tr_ctorSetMetainfoFromFile    ( tr_ctor        * ctor,
+                                         const char     * filename );
+
+int      tr_ctorSetMetainfoFromHash    ( tr_ctor        * ctor,
+                                         const char     * hashString );
+
+void     tr_ctorSetMaxConnectedPeers   ( tr_ctor        * ctor,
+                                         tr_ctorMode      mode,
+                                         uint16_t         maxConnectedPeers );
+
+void     tr_ctorSetDestination         ( tr_ctor        * ctor,
+                                         tr_ctorMode      mode,
+                                         const char     * directory );
+
+void     tr_ctorSetPaused              ( tr_ctor        * ctor,
+                                         tr_ctorMode      mode,
+                                         uint8_t          isPaused );
+
+int      tr_ctorGetMaxConnectedPeers   ( const tr_ctor  * ctor,
+                                         tr_ctorMode      mode,
+                                         uint16_t       * setmeCount );
+
+int      tr_ctorGetIsPaused            ( const tr_ctor  * ctor,
+                                         tr_ctorMode      mode,
+                                         uint8_t        * setmeIsPaused );
+
+int      tr_ctorGetDestination         ( const tr_ctor  * ctor,
+                                         tr_ctorMode      mode,
+                                         const char    ** setmeDestination );
+
+int      tr_ctorGetMetainfo            ( const tr_ctor  * ctor,
+                                         const struct benc_val_s ** setme );
+
+int      tr_ctorGetSave                ( const tr_ctor  * ctor );
+
+typedef struct tr_info tr_info;
+
+/**
+ * Parses the specified metainfo.
+ * Returns TR_OK if it parsed and can be added to Transmission.
+ * Returns TR_INVALID if it couldn't be parsed.
+ * Returns TR_EDUPLICATE if it parsed but can't be added. 
+ *     "destination" must be set to test for TR_EDUPLICATE.
+ *
+ * "setme_info" can be NULL if you don't need the information.
+ * If the metainfo can be parsed and setme_info is non-NULL,
+ * it will be filled with the metainfo's info.  You'll need to
+ * call tr_metainfoFree( setme_info ) when done with it.
+ */
+int tr_torrentParse( const tr_handle  * handle,
+                     const tr_ctor    * ctor,
+                     tr_info          * setme_info );
+
+/**
+ * Instantiate a single torrent.
+ */
+#define TR_EINVALID     1
+#define TR_EUNSUPPORTED 2
+#define TR_EDUPLICATE   3
+#define TR_EOTHER       666
+tr_torrent * tr_torrentNew( tr_handle      * handle,
+                            const tr_ctor  * ctor,
+                            int            * setmeError );
+
 
 /**
  *  Load all the torrents in tr_getTorrentsDirectory().
@@ -307,124 +465,24 @@ void tr_close( tr_handle * );
  *  from the previous session.
  */
 tr_torrent ** tr_loadTorrents ( tr_handle  * h,
-                                const char   * fallback_destination,
-                                int            isPaused,
-                                int          * setmeCount );
+                                tr_ctor    * ctor,
+                                int        * setmeCount );
 
 
-/***********************************************************************
- * tr_torrentInit
- ***********************************************************************
- * Opens and parses torrent file at 'path'. If the file exists and is
- * a valid torrent file, returns an handle and adds it to the list of
- * torrents (but doesn't start it). Returns NULL and sets *error
- * otherwise. If hash is not NULL and the torrent is already loaded
- * then it's 20-byte hash will be copied in. If the TR_FLAG_SAVE flag
- * is passed then a copy of the torrent file will be saved.
- **********************************************************************/
-#define TR_EINVALID     1
-#define TR_EUNSUPPORTED 2
-#define TR_EDUPLICATE   3
-#define TR_EOTHER       666
-tr_torrent * tr_torrentInit( tr_handle    * handle,
-                             const char   * metainfo_filename,
-                             const char   * destination,
-                             int            isPaused,
-                             int          * setme_error );
-
-/* This is a stupid hack to fix #415.  Probably I should fold all
- * these torrent constructors into a single function that takes
- * a function object to hold all these esoteric arguments. */
-tr_torrent * tr_torrentLoad( tr_handle    * handle,
-                             const char   * metainfo_filename,
-                             const char   * fallback_destination,
-                             int            isPaused,
-                             int          * setme_error );
-
-typedef struct tr_info tr_info;
 
 /**
- * Parses the specified metainfo file.
- *
- * Returns TR_OK if it parsed and can be added to Transmission.
- * Returns TR_INVALID if it couldn't be parsed.
- * Returns TR_EDUPLICATE if it parsed but can't be added.
- *
- * "destination" can be NULL if you don't need to know whether
- * or not the torrent can be added.
- *
- " "setme_info" can be NULL if you don't need the information.
- * If the metainfo can be parsed and setme_info is non-NULL,
- * it will be filled with the metadata's info.  You'll need to
- * call tr_metainfoFree( setme_info ) when done with it.
+ * Set whether or not torrents are allowed to do peer exchanges.
+ * By default, PEX is enabled.
+ * PEX is always disabled in private torrents regardless of this.
  */
-int tr_torrentParse( const tr_handle  * handle,
-                     const char       * metainfo_filename,
-                     const char       * destination,
-                     tr_info          * setme_info );
+void tr_setPexEnabled( tr_handle *, int isEnabled );
 
-/**
- * Parses the cached metainfo file that matches the given hash string.
- * See tr_torrentParse() for a description of the arguments
- */
-int
-tr_torrentParseHash( const tr_handle  * h,
-                     const char       * hashStr,
-                     const char       * destination,
-                     tr_info          * setme_info );
-
-
-/***********************************************************************
- * tr_torrentInitData
- ***********************************************************************
- * Like tr_torrentInit, except the actual torrent data is passed in
- * instead of the filename.
- **********************************************************************/
-tr_torrent * tr_torrentInitData( tr_handle *,
-                                 const uint8_t * data, size_t size,
-                                 const char * destination,
-                                 int isPaused,
-                                 int * error );
-
-/***********************************************************************
- * tr_torrentInitSaved
- ***********************************************************************
- * Opens and parses a torrent file as with tr_torrentInit, only taking
- * the hash string of a saved torrent file instead of a filename. There
- * are currently no valid flags for this function.
- **********************************************************************/
-tr_torrent * tr_torrentInitSaved( tr_handle *,
-                                  const char * hashStr,
-                                  const char * destination,
-                                  int isPaused,
-                                  int * error );
-
-/***********************************************************************
- * tr_torrentDisablePex
- ***********************************************************************
- * Disable or enable peer exchange for this torrent. Peer exchange is
- * enabled by default, except for private torrents where pex is
- * disabled and cannot be enabled.
- **********************************************************************/
-void tr_torrentDisablePex( tr_torrent *, int disable );
-int tr_torrentIsPexEnabled( const tr_torrent * );
+int tr_isPexEnabled( const tr_handle * );
 
 const tr_info * tr_torrentInfo( const tr_torrent * );
 
-#if 0
-/***********************************************************************
- * tr_torrentScrape
- ***********************************************************************
- * Asks the tracker for the count of seeders and leechers. Returns 0
- * and fills 's' and 'l' if successful. Otherwise returns 1 if the
- * tracker doesn't support the scrape protocol, is unreachable or
- * replied with some error. tr_torrentScrape may block up to 20 seconds
- * before returning.
- **********************************************************************/
-int tr_torrentScrape( tr_torrent *, int * s, int * l, int * d );
-#endif
-
 void   tr_torrentSetFolder( tr_torrent *, const char * );
+
 const char * tr_torrentGetFolder( const tr_torrent * );
 
 /***********************************************************************
@@ -500,6 +558,7 @@ int tr_torrentCanManualUpdate( const tr_torrent * );
  **********************************************************************/
 typedef struct tr_stat tr_stat;
 const tr_stat * tr_torrentStat( tr_torrent * );
+const tr_stat * tr_torrentStatCached( tr_torrent * );
 
 /***********************************************************************
  * tr_torrentPeers
@@ -528,19 +587,17 @@ void tr_torrentAmountFinished( const tr_torrent * tor, float * tab, int size );
 /***********************************************************************
  * tr_torrentRemoveSaved
  ***********************************************************************
- * Removes the saved copy of a torrent file for torrents which the
- * TR_FLAG_SAVE flag is set.
+ * delete's Transmission's copy of the torrent's metadata from
+ * tr_getTorrentsDirectory().
  **********************************************************************/
 void tr_torrentRemoveSaved( tr_torrent * );
 
 void tr_torrentRecheck( tr_torrent * );
 
-/***********************************************************************
- * tr_torrentClose
- ***********************************************************************
- * Frees memory allocated by tr_torrentInit. If the torrent was running,
- * it is stopped first.
- **********************************************************************/
+/**
+ * Frees memory allocated by tr_torrentNew().
+ * Running torrents are stopped first.
+ */
 void tr_torrentClose( tr_torrent * );
 
 /***********************************************************************
@@ -567,6 +624,15 @@ typedef struct tr_piece
 }
 tr_piece;
     
+typedef struct tr_tracker_info
+{
+    char * address;
+    int    port;
+    char * announce;
+    char * scrape;
+}
+tr_tracker_info;
+
 struct tr_info
 {
     /* Path to torrent */
@@ -684,6 +750,8 @@ struct tr_stat
     float               swarmspeed;
 
 #define TR_RATIO_NA  -1
+#define TR_RATIO_INF -2
+    /* TR_RATIO_INF, TR_RATIO_NA, or a regular ratio */
     float               ratio;
     
     uint64_t            startDate;
@@ -696,15 +764,32 @@ struct tr_file_stat
     float progress;
 };
 
+typedef enum
+{
+    TR_PEER_STATUS_HANDSHAKE             = (1<<0), /* we're handshaking with peer */
+
+    TR_PEER_STATUS_PEER_IS_SENDING       = (1<<1), /* peer is sending data to us */
+    TR_PEER_STATUS_PEER_IS_INTERESTED    = (1<<2), /* we have data the peer wants */
+    TR_PEER_STATUS_PEER_IS_CHOKED        = (1<<3), /* we refuse to send data to the peer */
+
+    TR_PEER_STATUS_CLIENT_IS_SENDING     = (1<<4), /* we're sending data to the peer */
+    TR_PEER_STATUS_CLIENT_SENT_REQUEST   = (1<<5), /* we've sent the peer a request */
+    TR_PEER_STATUS_CLIENT_IS_INTERESTED  = (1<<6), /* peer has data that we want */
+    TR_PEER_STATUS_CLIENT_IS_CHOKED      = (1<<7), /* peer refuses to send data to us */
+}
+tr_peer_status;
+
 struct tr_peer_stat
 {
-    char    addr[INET_ADDRSTRLEN];
-    const char * client;
+    char addr[INET_ADDRSTRLEN];
+    char client[80];
     
     unsigned int isEncrypted   : 1;
     unsigned int isDownloading : 1;
     unsigned int isUploading   : 1;
 
+    tr_peer_status status;
+
     uint8_t  from;
     uint16_t port;
     
@@ -718,28 +803,24 @@ struct tr_msg_list
     uint8_t              level;
     time_t               when;
     char               * message;
+    const char         * file;
+    int                  line;
     struct tr_msg_list * next;
 };
 
-struct tr_tracker_info
+typedef enum
 {
-    char * address;
-    int    port;
-    char * announce;
-    char * scrape;
-};
+    TR_NAT_TRAVERSAL_MAPPING,
+    TR_NAT_TRAVERSAL_MAPPED,
+    TR_NAT_TRAVERSAL_UNMAPPING,
+    TR_NAT_TRAVERSAL_UNMAPPED,
+    TR_NAT_TRAVERSAL_ERROR,
+}
+tr_nat_traversal_status;
 
 struct tr_handle_status
 {
-#define TR_NAT_TRAVERSAL_MAPPING        1
-#define TR_NAT_TRAVERSAL_MAPPED         2
-#define TR_NAT_TRAVERSAL_NOTFOUND       3
-#define TR_NAT_TRAVERSAL_ERROR          4
-#define TR_NAT_TRAVERSAL_UNMAPPING      5
-#define TR_NAT_TRAVERSAL_DISABLED       6
-#define TR_NAT_TRAVERSAL_IS_DISABLED( st ) \
-  ( TR_NAT_TRAVERSAL_DISABLED == (st) || TR_NAT_TRAVERSAL_UNMAPPING == (st) )
-    int natTraversalStatus;
+    tr_nat_traversal_status natTraversalStatus;
     int publicPort;
 };
 
