@@ -69,17 +69,22 @@ class Struct:
 
     def PrintDeclaration(self, file):
         print >>file, '/* Structure declaration for %s */' % self._name
-        print >>file, 'struct %s {' % self._name
+        print >>file, 'struct %s_access_ {' % self._name
         for entry in self._entries:
-            dcl = entry.Declaration()
-            dcl.extend(
-                entry.AssignDeclaration('(*%s_assign)' % entry.Name()))
+            dcl = entry.AssignDeclaration('(*%s_assign)' % entry.Name())
             dcl.extend(
                 entry.GetDeclaration('(*%s_get)' % entry.Name()))
             if entry.Array():
                 dcl.extend(
                     entry.AddDeclaration('(*%s_add)' % entry.Name()))
             self.PrintIdented(file, '  ', dcl)
+        print >>file, '};\n'
+
+        print >>file, 'struct %s {' % self._name
+        print >>file, '  struct %s_access_ *base;\n' % self._name
+        for entry in self._entries:
+            dcl = entry.Declaration()
+            self.PrintIdented(file, '  ', dcl)
         print >>file, ''
         for entry in self._entries:
             print >>file, '  uint8_t %s_set;' % entry.Name()
@@ -92,9 +97,9 @@ void %(name)s_clear(struct %(name)s *);
 void %(name)s_marshal(struct evbuffer *, const struct %(name)s *);
 int %(name)s_unmarshal(struct %(name)s *, struct evbuffer *);
 int %(name)s_complete(struct %(name)s *);
-void evtag_marshal_%(name)s(struct evbuffer *, uint8_t, 
+void evtag_marshal_%(name)s(struct evbuffer *, uint32_t, 
     const struct %(name)s *);
-int evtag_unmarshal_%(name)s(struct evbuffer *, uint8_t,
+int evtag_unmarshal_%(name)s(struct evbuffer *, uint32_t,
     struct %(name)s *);""" % { 'name' : self._name }
 
 
@@ -115,6 +120,13 @@ int evtag_unmarshal_%(name)s(struct evbuffer *, uint8_t,
                        ' * Implementation of %s\n'
                        ' */\n') % self._name
 
+        print >>file, \
+              'static struct %(name)s_access_ __%(name)s_base = {' % \
+              { 'name' : self._name }
+        for entry in self._entries:
+            self.PrintIdented(file, '  ', entry.CodeBase())
+        print >>file, '};\n'
+
         # Creation
         print >>file, (
             'struct %(name)s *\n'
@@ -124,7 +136,8 @@ int evtag_unmarshal_%(name)s(struct evbuffer *, uint8_t,
             '  if ((tmp = malloc(sizeof(struct %(name)s))) == NULL) {\n'
             '    event_warn("%%s: malloc", __func__);\n'
             '    return (NULL);\n'
-            '  }') % { 'name' : self._name }
+            '  }\n'
+            '  tmp->base = &__%(name)s_base;\n') % { 'name' : self._name }
 
         for entry in self._entries:
             self.PrintIdented(file, '  ', entry.CodeNew('tmp'))
@@ -196,7 +209,7 @@ int evtag_unmarshal_%(name)s(struct evbuffer *, uint8_t,
                        '%(name)s_unmarshal(struct %(name)s *tmp, '
                        ' struct evbuffer *evbuf)\n'
                        '{\n'
-                       '  uint8_t tag;\n'
+                       '  uint32_t tag;\n'
                        '  while (EVBUFFER_LENGTH(evbuf) > 0) {\n'
                        '    if (evtag_peek(evbuf, &tag) == -1)\n'
                        '      return (-1);\n'
@@ -247,9 +260,9 @@ int evtag_unmarshal_%(name)s(struct evbuffer *, uint8_t,
         print >>file, (
             'int\n'
             'evtag_unmarshal_%(name)s(struct evbuffer *evbuf, '
-            'uint8_t need_tag, struct %(name)s *msg)\n'
+            'uint32_t need_tag, struct %(name)s *msg)\n'
             '{\n'
-            '  uint8_t tag;\n'
+            '  uint32_t tag;\n'
             '  int res = -1;\n'
             '\n'
             '  struct evbuffer *tmp = evbuffer_new();\n'
@@ -271,7 +284,7 @@ int evtag_unmarshal_%(name)s(struct evbuffer *, uint8_t,
         # Complete message marshaling
         print >>file, (
             'void\n'
-            'evtag_marshal_%(name)s(struct evbuffer *evbuf, uint8_t tag, '
+            'evtag_marshal_%(name)s(struct evbuffer *evbuf, uint32_t tag, '
             'const struct %(name)s *msg)\n'
             '{\n'
             '  struct evbuffer *_buf = evbuffer_new();\n'
@@ -398,19 +411,16 @@ class Entry:
     def CodeFree(self, name):
         return []
 
-    def CodeNew(self, name):
+    def CodeBase(self):
         code = [
-            '%(vname)s->%(name)s_assign = %(parent_name)s_%(name)s_assign;',
-            '%(vname)s->%(name)s_get = %(parent_name)s_%(name)s_get;'
+            '%(parent_name)s_%(name)s_assign,',
+            '%(parent_name)s_%(name)s_get,'
             ]
         if self.Array():
-            code.append(
-                '%(vname)s->%(name)s_add = %(parent_name)s_%(name)s_add;')
+            code.append('%(parent_name)s_%(name)s_add,')
 
         code = '\n'.join(code)
-        trans = self.GetTranslation()
-        trans['vname'] = name
-        code = code % trans
+        code = code % self.GetTranslation()
         return code.split('\n')
 
     def Verify(self):
@@ -436,7 +446,7 @@ class EntryBytes(Entry):
         Entry.__init__(self, type, name, tag)
 
         self._length = length
-        self._ctype = 'uint8_t'
+        self._ctype = 'uint32_t'
 
     def GetDeclaration(self, funcname):
         code = [ 'int %s(struct %s *, %s **);' % (
@@ -545,6 +555,10 @@ class EntryInt(Entry):
 
         return dcl
 
+    def CodeNew(self, name):
+        code = ['%s->%s_data = 0;' % (name, self._name)]
+        return code
+
 class EntryString(Entry):
     def __init__(self, type, name, tag):
         # Init base class
@@ -595,7 +609,6 @@ class EntryString(Entry):
         
     def CodeNew(self, name):
         code  = ['%s->%s_data = NULL;' % (name, self._name)]
-        code.extend(Entry.CodeNew(self, name))
         return code
 
     def CodeFree(self, name):
@@ -721,7 +734,6 @@ class EntryStruct(Entry):
         
     def CodeNew(self, name):
         code  = ['%s->%s_data = NULL;' % (name, self._name)]
-        code.extend(Entry.CodeNew(self, name))
         return code
 
     def CodeFree(self, name):
@@ -828,7 +840,6 @@ class EntryVarBytes(Entry):
     def CodeNew(self, name):
         code  = ['%s->%s_data = NULL;' % (name, self._name),
                  '%s->%s_length = 0;' % (name, self._name) ]
-        code.extend(Entry.CodeNew(self, name))
         return code
 
     def CodeFree(self, name):
@@ -1011,7 +1022,6 @@ if (evtag_unmarshal_%(refname)s(%(buf)s, %(tag_name)s,
         code  = ['%s->%s_data = NULL;' % (name, self._name),
                  '%s->%s_length = 0;' % (name, self._name),
                  '%s->%s_num_allocated = 0;' % (name, self._name)]
-        code.extend(Entry.CodeNew(self, name))
         return code
 
     def CodeFree(self, name):
@@ -1099,10 +1109,10 @@ def ProcessOneEntry(newstruct, entry):
 
         if not tag_set:
             tag_set = 1
-            if not re.match(r'^[0-9]+$', token):
+            if not re.match(r'^(0x)?[0-9]+$', token):
                 print >>sys.stderr, 'Expected tag number: \"%s\"' % entry
                 sys.exit(1)
-            tag = int(token)
+            tag = int(token, 0)
             continue
 
         print >>sys.stderr, 'Cannot parse \"%s\"' % entry
@@ -1306,10 +1316,10 @@ def HeaderPreamble(name):
     pre += (
         '#define EVTAG_HAS(msg, member) ((msg)->member##_set == 1)\n'
         '#define EVTAG_ASSIGN(msg, member, args...) '
-        '(*(msg)->member##_assign)(msg, ## args)\n'
+        '(*(msg)->base->member##_assign)(msg, ## args)\n'
         '#define EVTAG_GET(msg, member, args...) '
-        '(*(msg)->member##_get)(msg, ## args)\n'
-        '#define EVTAG_ADD(msg, member) (*(msg)->member##_add)(msg)\n'
+        '(*(msg)->base->member##_get)(msg, ## args)\n'
+        '#define EVTAG_ADD(msg, member) (*(msg)->base->member##_add)(msg)\n'
         '#define EVTAG_LEN(msg, member) ((msg)->member##_length)\n'
         )
 
