@@ -35,7 +35,6 @@
 #endif
 
 #include <sys/types.h>
-#include <sys/tree.h>
 #include <sys/stat.h>
 #ifdef HAVE_SYS_TIME_H
 #include <sys/time.h>
@@ -43,6 +42,7 @@
 #include <sys/queue.h>
 #ifndef WIN32
 #include <sys/socket.h>
+#include <sys/wait.h>
 #include <sys/signal.h>
 #include <unistd.h>
 #include <netdb.h>
@@ -86,7 +86,7 @@ static struct event_base *global_base;
 #define read(fd,buf,len) recv((fd),(buf),(len),0)
 #endif
 
-void
+static void
 simple_read_cb(int fd, short event, void *arg)
 {
 	char buf[256];
@@ -105,7 +105,7 @@ simple_read_cb(int fd, short event, void *arg)
 	called++;
 }
 
-void
+static void
 simple_write_cb(int fd, short event, void *arg)
 {
 	int len;
@@ -117,7 +117,7 @@ simple_write_cb(int fd, short event, void *arg)
 		test_ok = 1;
 }
 
-void
+static void
 multiple_write_cb(int fd, short event, void *arg)
 {
 	struct event *ev = arg;
@@ -150,7 +150,7 @@ multiple_write_cb(int fd, short event, void *arg)
 	}
 }
 
-void
+static void
 multiple_read_cb(int fd, short event, void *arg)
 {
 	struct event *ev = arg;
@@ -172,17 +172,17 @@ multiple_read_cb(int fd, short event, void *arg)
 	}
 }
 
-void
+static void
 timeout_cb(int fd, short event, void *arg)
 {
 	struct timeval tv;
 	int diff;
 
 	gettimeofday(&tcalled, NULL);
-	if (timercmp(&tcalled, &tset, >))
-		timersub(&tcalled, &tset, &tv);
+	if (evutil_timercmp(&tcalled, &tset, >))
+		evutil_timersub(&tcalled, &tset, &tv);
 	else
-		timersub(&tset, &tcalled, &tv);
+		evutil_timersub(&tset, &tcalled, &tv);
 
 	diff = tv.tv_sec*1000 + tv.tv_usec/1000 - SECONDS * 1000;
 	if (diff < 0)
@@ -192,7 +192,13 @@ timeout_cb(int fd, short event, void *arg)
 		test_ok = 1;
 }
 
-void
+static void
+signal_cb_sa(int sig)
+{
+	test_ok = 2;
+}
+
+static void
 signal_cb(int fd, short event, void *arg)
 {
 	struct event *ev = arg;
@@ -206,7 +212,7 @@ struct both {
 	int nread;
 };
 
-void
+static void
 combined_read_cb(int fd, short event, void *arg)
 {
 	struct both *both = arg;
@@ -224,7 +230,7 @@ combined_read_cb(int fd, short event, void *arg)
 		exit(1);
 }
 
-void
+static void
 combined_write_cb(int fd, short event, void *arg)
 {
 	struct both *both = arg;
@@ -250,8 +256,8 @@ combined_write_cb(int fd, short event, void *arg)
 
 /* Test infrastructure */
 
-int
-setup_test(char *name)
+static int
+setup_test(const char *name)
 {
 
 	fprintf(stdout, "%s", name);
@@ -274,7 +280,7 @@ setup_test(char *name)
 	return (0);
 }
 
-int
+static int
 cleanup_test(void)
 {
 #ifndef WIN32
@@ -290,11 +296,11 @@ cleanup_test(void)
 		fprintf(stdout, "FAILED\n");
 		exit(1);
 	}
-
+        test_ok = 0;
 	return (0);
 }
 
-void
+static void
 test_simpleread(void)
 {
 	struct event ev;
@@ -313,7 +319,7 @@ test_simpleread(void)
 	cleanup_test();
 }
 
-void
+static void
 test_simplewrite(void)
 {
 	struct event ev;
@@ -329,7 +335,7 @@ test_simplewrite(void)
 	cleanup_test();
 }
 
-void
+static void
 test_multiple(void)
 {
 	struct event ev, ev2;
@@ -358,7 +364,7 @@ test_multiple(void)
 	cleanup_test();
 }
 
-void
+static void
 test_persistent(void)
 {
 	struct event ev, ev2;
@@ -387,7 +393,7 @@ test_persistent(void)
 	cleanup_test();
 }
 
-void
+static void
 test_combined(void)
 {
 	struct both r1, r2, w1, w2;
@@ -422,7 +428,7 @@ test_combined(void)
 	cleanup_test();
 }
 
-void
+static void
 test_simpletimeout(void)
 {
 	struct timeval tv;
@@ -442,7 +448,62 @@ test_simpletimeout(void)
 }
 
 #ifndef WIN32
-void
+extern struct event_base *current_base;
+static void
+test_fork(void)
+{
+	int status;
+	struct event ev;
+	pid_t pid;
+
+	setup_test("After fork: ");
+
+	write(pair[0], TEST1, strlen(TEST1)+1);
+
+	event_set(&ev, pair[1], EV_READ, simple_read_cb, &ev);
+	if (event_add(&ev, NULL) == -1)
+		exit(1);
+
+	if ((pid = fork()) == 0) {
+		/* in the child */
+		if (event_reinit(current_base) == -1) {
+			fprintf(stderr, "FAILED (reinit)\n");
+			exit(1);
+		}
+
+		event_dispatch();
+
+		/* we do not send an EOF; simple_read_cb requires an EOF 
+		 * to set test_ok.  we just verify that the callback was
+		 * called. */
+		exit(test_ok != 0 || called != 2);
+	}
+
+	/* wait for the child to read the data */
+	sleep(1);
+
+	write(pair[0], TEST1, strlen(TEST1)+1);
+
+	if (waitpid(pid, &status, 0) == -1) {
+		fprintf(stderr, "FAILED (fork)\n");
+		exit(1);
+	}
+	
+	if (WEXITSTATUS(status) != 0) {
+		fprintf(stderr, "FAILED (exit)\n");
+		exit(1);
+	}
+
+	/* test that the current event loop still works */
+	write(pair[0], TEST1, strlen(TEST1)+1);
+	shutdown(pair[0], SHUT_WR);
+
+	event_dispatch();
+
+	cleanup_test();
+}
+
+static void
 test_simplesignal(void)
 {
 	struct event ev;
@@ -465,11 +526,12 @@ test_simplesignal(void)
 	cleanup_test();
 }
 
-void
+static void
 test_immediatesignal(void)
 {
 	struct event ev;
 
+	test_ok = 0;
 	printf("Immediate signal: ");
 	signal_set(&ev, SIGUSR1, signal_cb, &ev);
 	signal_add(&ev, NULL);
@@ -479,7 +541,7 @@ test_immediatesignal(void)
 	cleanup_test();
 }
 
-void
+static void
 test_signal_dealloc(void)
 {
 	/* make sure that signal_event is event_del'ed and pipe closed */
@@ -490,15 +552,18 @@ test_signal_dealloc(void)
 	signal_add(&ev, NULL);
 	signal_del(&ev);
 	event_base_free(base);
+        /* If we got here without asserting, we're fine. */
+        test_ok = 1;
 	cleanup_test();
 }
 
-void
+static void
 test_signal_pipeloss(void)
 {
 	/* make sure that the base1 pipe is closed correctly. */
 	struct event_base *base1, *base2;
 	int pipe1;
+	test_ok = 0;
 	printf("Signal pipeloss: ");
 	base1 = event_init();
 	pipe1 = base1->sig.ev_signal_pair[0];
@@ -518,17 +583,19 @@ test_signal_pipeloss(void)
 /*
  * make two bases to catch signals, use both of them.  this only works
  * for event mechanisms that use our signal pipe trick.  kqueue handles
- * signals internally, and it looks like the first kqueue always gets the
- * signal.
+ * signals internally, and all interested kqueues get all the signals.
  */
-void
+static void
 test_signal_switchbase(void)
 {
 	struct event ev1, ev2;
 	struct event_base *base1, *base2;
+        int is_kqueue;
+	test_ok = 0;
 	printf("Signal switchbase: ");
 	base1 = event_init();
 	base2 = event_init();
+        is_kqueue = !strcmp(event_get_method(),"kqueue");
 	signal_set(&ev1, SIGUSR1, signal_cb, &ev1);
 	signal_set(&ev2, SIGUSR1, signal_cb, &ev2);
 	if (event_base_set(base1, &ev1) ||
@@ -543,22 +610,137 @@ test_signal_switchbase(void)
 	/* can handle signal before loop is called */
 	raise(SIGUSR1);
 	event_base_loop(base2, EVLOOP_NONBLOCK);
+        if (is_kqueue) {
+                if (!test_ok)
+                        goto done;
+                test_ok = 0;
+        }
 	event_base_loop(base1, EVLOOP_NONBLOCK);
-	if (test_ok) {
+	if (test_ok && !is_kqueue) {
 		test_ok = 0;
+
 		/* set base1 to handle signals */
 		event_base_loop(base1, EVLOOP_NONBLOCK);
 		raise(SIGUSR1);
 		event_base_loop(base1, EVLOOP_NONBLOCK);
 		event_base_loop(base2, EVLOOP_NONBLOCK);
 	}
+ done:
 	event_base_free(base1);
 	event_base_free(base2);
 	cleanup_test();
 }
+
+/*
+ * assert that a signal event removed from the event queue really is
+ * removed - with no possibility of it's parent handler being fired.
+ */
+static void
+test_signal_assert(void)
+{
+	struct event ev;
+	struct event_base *base = event_init();
+	test_ok = 0;
+	printf("Signal handler assert: ");
+	/* use SIGCONT so we don't kill ourselves when we signal to nowhere */
+	signal_set(&ev, SIGCONT, signal_cb, &ev);
+	signal_add(&ev, NULL);
+	/*
+	 * if signal_del() fails to reset the handler, it's current handler
+	 * will still point to evsignal_handler().
+	 */
+	signal_del(&ev);
+
+	raise(SIGCONT);
+	/* only way to verify we were in evsignal_handler() */
+	if (base->sig.evsignal_caught)
+		test_ok = 0;
+	else
+		test_ok = 1;
+
+	event_base_free(base);
+	cleanup_test();
+	return;
+}
+
+/*
+ * assert that we restore our previous signal handler properly.
+ */
+static void
+test_signal_restore(void)
+{
+	struct event ev;
+	struct event_base *base = event_init();
+#ifdef HAVE_SIGACTION
+	struct sigaction sa;
 #endif
 
-void
+	test_ok = 0;
+	printf("Signal handler restore: ");
+#ifdef HAVE_SIGACTION
+	sa.sa_handler = signal_cb_sa;
+	sa.sa_flags = 0x0;
+	sigemptyset(&sa.sa_mask);
+	if (sigaction(SIGUSR1, &sa, NULL) == -1)
+		goto out;
+#else
+	if (signal(SIGUSR1, signal_cb_sa) == SIG_ERR)
+		goto out;
+#endif
+	signal_set(&ev, SIGUSR1, signal_cb, &ev);
+	signal_add(&ev, NULL);
+	signal_del(&ev);
+
+	raise(SIGUSR1);
+	/* 1 == signal_cb, 2 == signal_cb_sa, we want our previous handler */
+	if (test_ok != 2)
+		test_ok = 0;
+out:
+	event_base_free(base);
+	cleanup_test();
+	return;
+}
+#endif
+
+static void
+test_free_active_base(void)
+{
+	struct event_base *base1;
+	struct event ev1;
+	setup_test("Free active base: ");
+	base1 = event_init();
+	event_set(&ev1, pair[1], EV_READ, simple_read_cb, &ev1);
+	event_base_set(base1, &ev1);
+	event_add(&ev1, NULL);
+	/* event_del(&ev1); */
+	event_base_free(base1);
+	test_ok = 1;
+	cleanup_test();
+}
+
+static void
+test_event_base_new(void)
+{
+	struct event_base *base;
+	struct event ev1;
+	setup_test("Event base new: ");
+
+	write(pair[0], TEST1, strlen(TEST1)+1);
+	shutdown(pair[0], SHUT_WR);
+
+	base = event_base_new();
+	event_set(&ev1, pair[1], EV_READ, simple_read_cb, &ev1);
+	event_base_set(base, &ev1);
+	event_add(&ev1, NULL);
+
+	event_base_dispatch(base);
+
+	event_base_free(base);
+	test_ok = 1;
+	cleanup_test();
+}
+
+static void
 test_loopexit(void)
 {
 	struct timeval tv, tv_start, tv_end;
@@ -578,7 +760,7 @@ test_loopexit(void)
 	gettimeofday(&tv_start, NULL);
 	event_dispatch();
 	gettimeofday(&tv_end, NULL);
-	timersub(&tv_end, &tv_start, &tv_end);
+	evutil_timersub(&tv_end, &tv_start, &tv_end);
 
 	evtimer_del(&ev);
 
@@ -588,7 +770,70 @@ test_loopexit(void)
 	cleanup_test();
 }
 
-void
+static void
+test_loopexit_multiple(void)
+{
+	struct timeval tv;
+	struct event_base *base;
+
+	setup_test("Loop Multiple exit: ");
+
+	base = event_base_new();
+	
+	tv.tv_usec = 0;
+	tv.tv_sec = 1;
+	event_base_loopexit(base, &tv);
+
+	tv.tv_usec = 0;
+	tv.tv_sec = 2;
+	event_base_loopexit(base, &tv);
+
+	event_base_dispatch(base);
+
+	event_base_free(base);
+	
+	test_ok = 1;
+
+	cleanup_test();
+}
+
+static void
+break_cb(int fd, short events, void *arg)
+{
+	test_ok = 1;
+	event_loopbreak();
+}
+
+static void
+fail_cb(int fd, short events, void *arg)
+{
+	test_ok = 0;
+}
+
+static void
+test_loopbreak(void)
+{
+	struct event ev1, ev2;
+	struct timeval tv;
+
+	setup_test("Loop break: ");
+
+	tv.tv_sec = 0;
+	tv.tv_usec = 0;
+	evtimer_set(&ev1, break_cb, NULL);
+	evtimer_add(&ev1, &tv);
+	evtimer_set(&ev2, fail_cb, NULL);
+	evtimer_add(&ev2, &tv);
+
+	event_dispatch();
+
+	evtimer_del(&ev1);
+	evtimer_del(&ev2);
+
+	cleanup_test();
+}
+
+static void
 test_evbuffer(void) {
 
 	struct evbuffer *evb = evbuffer_new();
@@ -605,12 +850,12 @@ test_evbuffer(void) {
 	cleanup_test();
 }
 
-void
+static void
 test_evbuffer_find(void)
 {
 	u_char* p;
-	char* test1 = "1234567890\r\n";
-	char* test2 = "1234567890\r";
+	const char* test1 = "1234567890\r\n";
+	const char* test2 = "1234567890\r";
 #define EVBUFFER_INITIAL_LENGTH 256
 	char test3[EVBUFFER_INITIAL_LENGTH];
 	unsigned int i;
@@ -660,7 +905,7 @@ test_evbuffer_find(void)
 	evbuffer_free(buf);
 }
 
-void
+static void
 readcb(struct bufferevent *bev, void *arg)
 {
 	if (EVBUFFER_LENGTH(bev->input) == 8333) {
@@ -669,20 +914,20 @@ readcb(struct bufferevent *bev, void *arg)
 	}
 }
 
-void
+static void
 writecb(struct bufferevent *bev, void *arg)
 {
 	if (EVBUFFER_LENGTH(bev->output) == 0)
 		test_ok++;
 }
 
-void
+static void
 errorcb(struct bufferevent *bev, short what, void *arg)
 {
 	test_ok = -2;
 }
 
-void
+static void
 test_bufferevent(void)
 {
 	struct bufferevent *bev1, *bev2;
@@ -718,7 +963,7 @@ struct test_pri_event {
 	int count;
 };
 
-void
+static void
 test_priorities_cb(int fd, short what, void *arg)
 {
 	struct test_pri_event *pri = arg;
@@ -731,11 +976,11 @@ test_priorities_cb(int fd, short what, void *arg)
 
 	pri->count++;
 
-	timerclear(&tv);
+	evutil_timerclear(&tv);
 	event_add(&pri->ev, &tv);
 }
 
-void
+static void
 test_priorities(int npriorities)
 {
 	char buf[32];
@@ -762,7 +1007,7 @@ test_priorities(int npriorities)
 		exit(1);
 	}
 
-	timerclear(&tv);
+	evutil_timerclear(&tv);
 
 	if (event_add(&one.ev, &tv) == -1)
 		exit(1);
@@ -798,7 +1043,7 @@ test_multiple_cb(int fd, short event, void *arg)
 		test_ok |= 2;
 }
 
-void
+static void
 test_multiple_events_for_same_fd(void)
 {
    struct event e1, e2;
@@ -821,9 +1066,11 @@ test_multiple_events_for_same_fd(void)
    cleanup_test();
 }
 
-int decode_int(uint32_t *pnumber, struct evbuffer *evbuf);
+int evtag_decode_int(uint32_t *pnumber, struct evbuffer *evbuf);
+int evtag_encode_tag(struct evbuffer *evbuf, uint32_t number);
+int evtag_decode_tag(uint32_t *pnumber, struct evbuffer *evbuf);
 
-void
+static void
 read_once_cb(int fd, short event, void *arg)
 {
 	char buf[256];
@@ -842,7 +1089,7 @@ read_once_cb(int fd, short event, void *arg)
 	called++;
 }
 
-void
+static void
 test_want_only_once(void)
 {
 	struct event ev;
@@ -854,7 +1101,7 @@ test_want_only_once(void)
 	write(pair[0], TEST1, strlen(TEST1)+1);
 
 	/* Setup the loop termination */
-	timerclear(&tv);
+	evutil_timerclear(&tv);
 	tv.tv_sec = 1;
 	event_loopexit(&tv);
 	
@@ -868,7 +1115,7 @@ test_want_only_once(void)
 
 #define TEST_MAX_INT	6
 
-void
+static void
 evtag_int_test(void)
 {
 	struct evbuffer *tmp = evbuffer_new();
@@ -888,7 +1135,7 @@ evtag_int_test(void)
 	}
 
 	for (i = 0; i < TEST_MAX_INT; i++) {
-		if (decode_int(&integer, tmp) == -1) {
+		if (evtag_decode_int(&integer, tmp) == -1) {
 			fprintf(stderr, "decode %d failed", i);
 			exit(1);
 		}
@@ -908,7 +1155,7 @@ evtag_int_test(void)
 	fprintf(stdout, "\t%s: OK\n", __func__);
 }
 
-void
+static void
 evtag_fuzz(void)
 {
 	u_char buffer[4096];
@@ -935,7 +1182,7 @@ evtag_fuzz(void)
 
 	/* Now insert some corruption into the tag length field */
 	evbuffer_drain(tmp, -1);
-	timerclear(&tv);
+	evutil_timerclear(&tv);
 	tv.tv_sec = 1;
 	evtag_marshal_timeval(tmp, 0, &tv);
 	evbuffer_add(tmp, buffer, sizeof(buffer));
@@ -951,7 +1198,47 @@ evtag_fuzz(void)
 	fprintf(stdout, "\t%s: OK\n", __func__);
 }
 
-void
+static void
+evtag_tag_encoding(void)
+{
+	struct evbuffer *tmp = evbuffer_new();
+	uint32_t integers[TEST_MAX_INT] = {
+		0xaf0, 0x1000, 0x1, 0xdeadbeef, 0x00, 0xbef000
+	};
+	uint32_t integer;
+	int i;
+
+	for (i = 0; i < TEST_MAX_INT; i++) {
+		int oldlen, newlen;
+		oldlen = EVBUFFER_LENGTH(tmp);
+		evtag_encode_tag(tmp, integers[i]);
+		newlen = EVBUFFER_LENGTH(tmp);
+		fprintf(stdout, "\t\tencoded 0x%08x with %d bytes\n",
+		    integers[i], newlen - oldlen);
+	}
+
+	for (i = 0; i < TEST_MAX_INT; i++) {
+		if (evtag_decode_tag(&integer, tmp) == -1) {
+			fprintf(stderr, "decode %d failed", i);
+			exit(1);
+		}
+		if (integer != integers[i]) {
+			fprintf(stderr, "got %x, wanted %x",
+			    integer, integers[i]);
+			exit(1);
+		}
+	}
+
+	if (EVBUFFER_LENGTH(tmp) != 0) {
+		fprintf(stderr, "trailing data");
+		exit(1);
+	}
+	evbuffer_free(tmp);
+
+	fprintf(stdout, "\t%s: OK\n", __func__);
+}
+
+static void
 evtag_test(void)
 {
 	fprintf(stdout, "Testing Tagging:\n");
@@ -960,10 +1247,12 @@ evtag_test(void)
 	evtag_int_test();
 	evtag_fuzz();
 
+	evtag_tag_encoding();
+
 	fprintf(stdout, "OK\n");
 }
 
-void
+static void
 rpc_test(void)
 {
 	struct msg *msg, *msg2;
@@ -971,6 +1260,7 @@ rpc_test(void)
 	struct run *run;
 	struct evbuffer *tmp = evbuffer_new();
 	struct timeval tv_start, tv_end;
+	uint32_t tag;
 	int i;
 
 	fprintf(stdout, "Testing RPC: ");
@@ -1002,16 +1292,26 @@ rpc_test(void)
 		exit(1);
 	}
 
-	evtag_marshal_msg(tmp, 0, msg);
+	evtag_marshal_msg(tmp, 0xdeaf, msg);
+
+	if (evtag_peek(tmp, &tag) == -1) {
+		fprintf(stderr, "Failed to peak tag.\n");
+		exit (1);
+	}
+
+	if (tag != 0xdeaf) {
+		fprintf(stderr, "Got incorrect tag: %0x.\n", tag);
+		exit (1);
+	}
 
 	msg2 = msg_new();
-	if (evtag_unmarshal_msg(tmp, 0, msg2) == -1) {
+	if (evtag_unmarshal_msg(tmp, 0xdeaf, msg2) == -1) {
 		fprintf(stderr, "Failed to unmarshal message.\n");
 		exit(1);
 	}
 
 	gettimeofday(&tv_end, NULL);
-	timersub(&tv_end, &tv_start, &tv_end);
+	evutil_timersub(&tv_end, &tv_start, &tv_end);
 	fprintf(stderr, "(%.1f us/add) ",
 	    (float)tv_end.tv_sec/(float)i * 1000000.0 +
 	    tv_end.tv_usec / (float)i);
@@ -1036,6 +1336,32 @@ rpc_test(void)
 	fprintf(stdout, "OK\n");
 }
 
+static void
+test_evutil_strtoll(void)
+{
+        const char *s;
+        char *endptr;
+        setup_test("evutil_stroll: ");
+        test_ok = 0;
+
+        if (evutil_strtoll("5000000000", NULL, 10) != ((ev_int64_t)5000000)*1000)
+                goto err;
+        if (evutil_strtoll("-5000000000", NULL, 10) != ((ev_int64_t)5000000)*-1000)
+                goto err;
+        s = " 99999stuff";
+        if (evutil_strtoll(s, &endptr, 10) != (ev_int64_t)99999)
+                goto err;
+        if (endptr != s+6)
+                goto err;
+        if (evutil_strtoll("foo", NULL, 10) != 0)
+                goto err;
+
+        test_ok = 1;
+ err:
+        cleanup_test();
+}
+
+
 int
 main (int argc, char **argv)
 {
@@ -1054,6 +1380,8 @@ main (int argc, char **argv)
 	/* Initalize the event library */
 	global_base = event_init();
 
+        test_evutil_strtoll();
+
 	/* use the global event base and need to be called first */
 	test_priorities(1);
 	test_priorities(2);
@@ -1064,12 +1392,20 @@ main (int argc, char **argv)
 	
 	test_bufferevent();
 
+	test_free_active_base();
+
+	test_event_base_new();
+
 	http_suite();
 
 	rpc_suite();
 
 	dns_suite();
 	
+#ifndef WIN32
+	test_fork();
+#endif
+
 	test_simpleread();
 
 	test_simplewrite();
@@ -1086,7 +1422,10 @@ main (int argc, char **argv)
 	test_immediatesignal();
 #endif
 	test_loopexit();
+	test_loopbreak();
 
+	test_loopexit_multiple();
+	
 	test_multiple_events_for_same_fd();
 
 	test_want_only_once();
@@ -1099,6 +1438,8 @@ main (int argc, char **argv)
 	test_signal_dealloc();
 	test_signal_pipeloss();
 	test_signal_switchbase();
+	test_signal_restore();
+	test_signal_assert();
 #endif
 	
 	return (0);
