@@ -15,7 +15,7 @@
  * along with this program; if not, write to the Free Software
  * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
  *
- * $Id: xmission.cc 3426 2007-10-15 20:58:39Z charles $
+ * $Id: xmission.cc 4390 2007-12-31 01:19:35Z livings124 $
  */
 
 #include <ctype.h>
@@ -511,7 +511,7 @@ MyFrame :: MyFrame(const wxString& title, const wxPoint& pos, const wxSize& size
 
     long port;
     wxString key = _T("port");
-    if( !myConfig->Read( key, &port, 9090 ) )
+    if( !myConfig->Read( key, &port, TR_DEFAULT_PORT ) )
         myConfig->Write( key, port );
     tr_setBindPort( handle, port );
 
@@ -708,7 +708,7 @@ void MyFrame::OnAbout(wxCommandEvent& WXUNUSED(event))
     info.SetVersion(_T(LONG_VERSION_STRING));
     info.SetCopyright(_T("Copyright 2005-2007 The Transmission Project"));
     info.SetDescription(_T("A fast and easy BitTorrent client"));
-    info.SetWebSite( _T( "http://transmission.m0k.org/" ) );
+    info.SetWebSite( _T( "http://www.transmissionbt.com/" ) );
     info.SetIcon( ico );
     info.AddDeveloper( _T("Charles Kerr (Back-end, GTK+, wxWidgets)") );
     info.AddDeveloper( _T("Mitchell Livingston (Back-end; OS X)")  );
